@@ -19,6 +19,7 @@
  *  <http://www.gnu.org/licenses/>.                                         *
  ****************************************************************************/
 
+// THESE INCLUDES ARE IN RANDOM ORDER.  LET'S CLEAN IT UP AFTER RELEASE. - Gene
 #include "constants.h"
 #include "syscallwrappers.h"
 #include "connection.h"
@@ -109,6 +110,7 @@ static bool _isBlacklistedFile ( dmtcp::string& path )
 {
   if ((dmtcp::Util::strStartsWith(path, "/dev/") &&
        !dmtcp::Util::strStartsWith(path, "/dev/shm/")) ||
+      dmtcp::Util::strStartsWith(path, "/proc/") ||
       dmtcp::Util::strStartsWith(path, dmtcp::UniquePid::getTmpDir().c_str())) {
     return true;
   }
@@ -128,7 +130,9 @@ static bool _isBlacklistedTcp ( int sockfd,
     //However, libc.so:getpwuid() can call libnss_ldap.so which calls
     // libldap-2.4.so to create the LDAP socket while evading our connect
     // wrapper.
-    int blacklistedRemotePorts[] = {389, 636, -1}; /* LDAP ports */
+    int blacklistedRemotePorts[] = {53,                 // DNS Server
+                                    389, 636,           // LDAP
+                                    -1};
     int i;
     for (i = 0; blacklistedRemotePorts[i] != -1; i++) {
       if (ntohs(addr->sin_port) == blacklistedRemotePorts[i]) {
@@ -158,6 +162,12 @@ dmtcp::TcpConnection& dmtcp::Connection::asTcp()
   return * ( ( TcpConnection* ) 0 );
 }
 
+dmtcp::EpollConnection& dmtcp::Connection::asEpoll()
+{
+  JASSERT ( false ) ( _id ) ( _type ).Text ( "Invalid conversion." );
+  return * ( ( EpollConnection* ) 0 );
+}
+
 void dmtcp::Connection::restartDup2(int oldFd, int fd){
   errno = 0;
   JWARNING ( _real_dup2 ( oldFd, fd ) == fd ) ( oldFd ) ( fd ) ( JASSERT_ERRNO );
@@ -183,6 +193,13 @@ void dmtcp::Connection::restoreOptions ( const dmtcp::vector<int>& fds )
   JASSERT ( _fcntlOwner != -1 ) ( _fcntlOwner );
   JASSERT ( _fcntlSignal >= 0 ) ( _fcntlSignal );
   errno = 0;
+  if (this->conType() == EPOLL)
+  {
+    JTRACE("RGARG: For EPOLL");
+    //return;
+  }
+  else
+    JTRACE("RGARG: connection flags restoring" ) (this->conType());
   JASSERT ( fcntl ( fds[0], F_SETFL, _fcntlFlags ) == 0 ) ( fds[0] ) ( _fcntlFlags ) ( JASSERT_ERRNO );
 
   // FIXME: When restarting, the VirtualPidTable original to current pid
@@ -215,13 +232,12 @@ void dmtcp::Connection::doLocking ( const dmtcp::vector<int>& fds )
 /*onSocket*/
 dmtcp::TcpConnection::TcpConnection ( int domain, int type, int protocol )
   : Connection ( TCP_CREATED )
-#ifdef EXTERNAL_SOCKET_HANDLING
-  , _peerType ( PEER_UNKNOWN )
-#endif
   , _sockDomain ( domain )
   , _sockType ( type )
   , _sockProtocol ( protocol )
   , _listenBacklog ( -1 )
+  , _peerType ( PEER_UNKNOWN )
+  , _socketPairRestored ( false )
   , _bindAddrlen ( 0 )
   , _acceptRemoteId ( ConnectionIdentifier::Null() )
 {
@@ -287,13 +303,12 @@ void dmtcp::TcpConnection::onConnect( int sockfd,
 /*onAccept*/
 dmtcp::TcpConnection::TcpConnection ( const TcpConnection& parent, const ConnectionIdentifier& remote )
   : Connection ( TCP_ACCEPT )
-#ifdef EXTERNAL_SOCKET_HANDLING
-  , _peerType ( PEER_UNKNOWN )
-#endif
   , _sockDomain ( parent._sockDomain )
   , _sockType ( parent._sockType )
   , _sockProtocol ( parent._sockProtocol )
   , _listenBacklog ( -1 )
+  , _peerType ( PEER_UNKNOWN )
+  , _socketPairRestored ( false )
   , _bindAddrlen ( 0 )
   , _acceptRemoteId ( remote )
 {
@@ -405,57 +420,62 @@ void dmtcp::TcpConnection::preCheckpoint ( const dmtcp::vector<int>& fds
   }
 }
 
-void dmtcp::TcpConnection::doSendHandshakes( const dmtcp::vector<int>& fds, const dmtcp::UniquePid& coordinator ){
-    switch ( tcpType() )
-    {
-      case TCP_CONNECT:
-      case TCP_ACCEPT:
-        if ( hasLock ( fds ) )
-        {
-          JTRACE ("Sending handshake ...") (id()) (fds[0]);
-          jalib::JSocket sock(fds[0]);
-          sendHandshake( sock, coordinator );
+void dmtcp::TcpConnection::doSendHandshakes(const dmtcp::vector<int>& fds,
+                                            const dmtcp::UniquePid& coordinator)
+{
+  switch ( tcpType() )
+  {
+    case TCP_CONNECT:
+    case TCP_ACCEPT:
+      if ( hasLock ( fds ) )
+      {
+        JTRACE ("Sending handshake ...") (id()) (fds[0]);
+        jalib::JSocket sock(fds[0]);
+        sendHandshake( sock, coordinator );
+      }
+      else
+      {
+        if (really_verbose) {
+          JTRACE("Skipping handshake send (shared socket, not owner).")
+            (id()) (fds[0]);
         }
-        else
-        {
-          if (really_verbose) {
-            JTRACE("Skipping handshake send (shared socket, not owner).")
-		(id()) (fds[0]);
-          }
-        }
-        break;
-      case TCP_EXTERNAL_CONNECT:
-        JTRACE ( "Socket to External Process, skipping handshake send" ) ( fds[0] );
-        break;
-    }
+      }
+      break;
+    case TCP_EXTERNAL_CONNECT:
+      JTRACE ( "Socket to External Process, skipping handshake send" ) ( fds[0] );
+      break;
   }
-  void dmtcp::TcpConnection::doRecvHandshakes( const dmtcp::vector<int>& fds, const dmtcp::UniquePid& coordinator ){
-    switch ( tcpType() )
-    {
-      case TCP_CONNECT:
-      case TCP_ACCEPT:
-        if ( hasLock ( fds ) )
-        {
-          JTRACE ("Receiving handshake ...") (id()) (fds[0]);
-          jalib::JSocket sock(fds[0]);
-          recvHandshake( sock, coordinator );
-          if (really_verbose) {
-            JTRACE ("Received handshake.") (getRemoteId()) (fds[0]);
-          }
+}
+
+void dmtcp::TcpConnection::doRecvHandshakes(const dmtcp::vector<int>& fds,
+                                            const dmtcp::UniquePid& coordinator)
+{
+  switch ( tcpType() )
+  {
+    case TCP_CONNECT:
+    case TCP_ACCEPT:
+      if ( hasLock ( fds ) )
+      {
+        JTRACE ("Receiving handshake ...") (id()) (fds[0]);
+        jalib::JSocket sock(fds[0]);
+        recvHandshake( sock, coordinator );
+        if (really_verbose) {
+          JTRACE ("Received handshake.") (getRemoteId()) (fds[0]);
         }
-        else
-        {
-          if (really_verbose) {
-            JTRACE ("Skipping handshake recv (shared socket, not owner).")
-              (id()) (fds[0]);
-          }
+      }
+      else
+      {
+        if (really_verbose) {
+          JTRACE ("Skipping handshake recv (shared socket, not owner).")
+            (id()) (fds[0]);
         }
-        break;
-      case TCP_EXTERNAL_CONNECT:
-        JTRACE ( "Socket to External Process, skipping handshake recv" ) ( fds[0] );
-        break;
-    }
+      }
+      break;
+    case TCP_EXTERNAL_CONNECT:
+      JTRACE ( "Socket to External Process, skipping handshake recv" ) ( fds[0] );
+      break;
   }
+}
 
 void dmtcp::TcpConnection::postCheckpoint ( const dmtcp::vector<int>& fds, bool isRestart )
 {
@@ -467,7 +487,45 @@ void dmtcp::TcpConnection::postCheckpoint ( const dmtcp::vector<int>& fds, bool
     restoreOptions ( fds );
   }
 }
-void dmtcp::TcpConnection::restore ( const dmtcp::vector<int>& fds, ConnectionRewirer& rewirer )
+
+void dmtcp::TcpConnection::restoreSocketPair(const dmtcp::vector<int>& fds,
+                                             dmtcp::TcpConnection *peer,
+                                             const dmtcp::vector<int>& peerfds)
+{
+  int sv[2];
+  JASSERT(_peerType == PEER_SOCKETPAIR && _socketpairPeerId == peer->id())
+    (_peerType) (_socketpairPeerId) (peer->id());
+  JASSERT(fds.size() > 0);
+  JASSERT(peerfds.size() > 0);
+
+  if (_socketPairRestored) {
+    _socketPairRestored = false;
+    return;
+  }
+  JASSERT(_real_socketpair( _sockDomain,_sockType,_sockProtocol, sv) == 0)
+    (JASSERT_ERRNO);
+
+  jalib::JSocket sock1(sv[0]);
+  jalib::JSocket sock2(sv[1]);
+
+  sock1.changeFd(fds[0]);
+  sock2.changeFd(peerfds[0]);
+
+  for (size_t i=1; i<fds.size(); ++i) {
+    JASSERT(_real_dup2(fds[0], fds[i]) == fds[i]) (fds[0]) (fds[i])
+      .Text("dup2() failed");
+  }
+
+  for (size_t i=1; i<peerfds.size(); ++i) {
+    JASSERT(_real_dup2(peerfds[0], peerfds[i]) == peerfds[i])
+      (peerfds[0]) (peerfds[i]) .Text("dup2() failed");
+  }
+  peer->_socketPairRestored = true;
+  JTRACE("Restored Socketpair") (id()) (peer->id()) (fds[0]) (peerfds[0]);
+}
+
+void dmtcp::TcpConnection::restore(const dmtcp::vector<int>& fds,
+                                   ConnectionRewirer *rewirer)
 {
   JASSERT ( fds.size() > 0 );
   switch ( tcpType() )
@@ -585,11 +643,11 @@ void dmtcp::TcpConnection::restore ( const dmtcp::vector<int>& fds, ConnectionRe
         .Text("Can't restore a TCP_ACCEPT socket with null acceptRemoteId.\n"
 	      "  Perhaps handshake went wrong?");
       JTRACE ( "registerOutgoing" ) ( id() ) ( _acceptRemoteId ) ( fds[0] );
-      rewirer.registerOutgoing ( _acceptRemoteId, fds );
+      rewirer->registerOutgoing ( _acceptRemoteId, fds );
       break;
     case TCP_CONNECT:
       JTRACE ( "registerIncoming" ) ( id() ) ( _acceptRemoteId ) ( fds[0] );
-      rewirer.registerIncoming ( id(), fds );
+      rewirer->registerIncoming ( id(), fds );
       break;
 //    case TCP_EXTERNAL_CONNECT:
 //      int sockFd = _real_socket ( _sockDomain, _sockType, _sockProtocol );
@@ -689,7 +747,8 @@ void dmtcp::PtyConnection::postCheckpoint ( const dmtcp::vector<int>& fds, bool
   restoreOptions ( fds );
 }
 
-void dmtcp::PtyConnection::restore ( const dmtcp::vector<int>& fds, ConnectionRewirer& rewirer )
+void dmtcp::PtyConnection::restore(const dmtcp::vector<int>& fds,
+                                   ConnectionRewirer*)
 {
   JASSERT ( fds.size() > 0 );
 
@@ -923,6 +982,22 @@ void dmtcp::PtyConnection::restoreOptions ( const dmtcp::vector<int>& fds )
 // Default 100MB
 #define MAX_FILESIZE_TO_AUTOCKPT (100 * 1024 * 1024)
 
+void dmtcp::FileConnection::doLocking(const dmtcp::vector<int>& fds)
+{
+  if (dmtcp::Util::strStartsWith(_path, "/proc/")) {
+    int index = 6;
+    char *rest;
+    pid_t proc_pid = strtol(&_path[index], &rest, 0);
+    if (proc_pid > 0 && *rest == '/') {
+      _type = FILE_PROCFS;
+      if (proc_pid != getpid()) {
+        return;
+      }
+    }
+  }
+  Connection::doLocking(fds);
+}
+
 void dmtcp::FileConnection::handleUnlinkedFile()
 {
   if (!jalib::Filesystem::FileExists(_path)) {
@@ -968,7 +1043,7 @@ void dmtcp::FileConnection::preCheckpoint ( const dmtcp::vector<int>& fds
 
   calculateRelativePath();
 
-  _ckptFilesDir = UniquePid::checkpointFilesSubDirName();
+  _ckptFilesDir = UniquePid::getCkptFilesSubDir();
 
   // Read the current file descriptor offset
   _offset = lseek(fds[0], 0, SEEK_CUR);
@@ -1031,6 +1106,15 @@ void dmtcp::FileConnection::refreshPath()
       _path = fullPath;
       JTRACE("Change _path based on relative path") (oldPath) (_path) (_rel_path);
     }
+  } else if (_type == FILE_PROCFS) {
+    int index = 6;
+    char *rest;
+    char buf[64];
+    pid_t proc_pid = strtol(&_path[index], &rest, 0);
+    if (proc_pid > 0 && *rest == '/') {
+      sprintf(buf, "/proc/%d/%s", _real_getpid(), rest);
+      _path = buf;
+    }
   }
 }
 
@@ -1042,8 +1126,8 @@ void dmtcp::FileConnection::restoreOptions ( const dmtcp::vector<int>& fds )
 }
 
 
-void dmtcp::FileConnection::restore ( const dmtcp::vector<int>& fds,
-                                      ConnectionRewirer& rewirer )
+void dmtcp::FileConnection::restore(const dmtcp::vector<int>& fds,
+                                    ConnectionRewirer*)
 {
   struct stat buf;
 
@@ -1405,7 +1489,8 @@ void dmtcp::FifoConnection::restoreOptions ( const dmtcp::vector<int>& fds )
 }
 
 
-void dmtcp::FifoConnection::restore ( const dmtcp::vector<int>& fds, ConnectionRewirer& rewirer )
+void dmtcp::FifoConnection::restore(const dmtcp::vector<int>& fds,
+                                    ConnectionRewirer*)
 {
   JASSERT ( fds.size() > 0 );
 
@@ -1451,8 +1536,8 @@ void dmtcp::Connection::serialize ( jalib::JBinarySerializer& o )
 void dmtcp::TcpConnection::serializeSubClass ( jalib::JBinarySerializer& o )
 {
   JSERIALIZE_ASSERT_POINT ( "dmtcp::TcpConnection" );
-  o & _sockDomain  & _sockType & _sockProtocol & _listenBacklog
-    & _bindAddrlen & _bindAddr & _acceptRemoteId;
+  o & _sockDomain  & _sockType & _sockProtocol & _listenBacklog & _peerType
+    & _bindAddrlen & _bindAddr & _acceptRemoteId & _socketpairPeerId;
 
   JSERIALIZE_ASSERT_POINT ( "SocketOptions:" );
   size_t numSockOpts = _sockOptions.size();
@@ -1763,7 +1848,9 @@ void dmtcp::StdioConnection::postCheckpoint ( const dmtcp::vector<int>& fds , bo
     restoreOptions ( fds );
   //nothing
 }
-void dmtcp::StdioConnection::restore ( const dmtcp::vector<int>& fds, ConnectionRewirer& ){
+void dmtcp::StdioConnection::restore(const dmtcp::vector<int>& fds,
+                                     ConnectionRewirer*)
+{
   for(size_t i=0; i<fds.size(); ++i){
     int fd = fds[i];
     if(fd <= 2){
@@ -1806,6 +1893,273 @@ void dmtcp::StdioConnection::mergeWith ( const Connection& that ){
 }
 
 void dmtcp::StdioConnection::restartDup2(int oldFd, int newFd){
-  static ConnectionRewirer ignored;
-  restore(dmtcp::vector<int>(1,newFd), ignored);
+  restore(dmtcp::vector<int>(1, newFd));
+}
+
+
+void dmtcp::EpollConnection::preCheckpoint ( const dmtcp::vector<int>& fds
+    , KernelBufferDrainer& drain )
+{
+  JASSERT ( fds.size() > 0 );
+  JTRACE ( "RGARG, RGARG, RGARG, RGARG" ) ( fds[0] ) ( id() );
+}
+
+void dmtcp::EpollConnection::postCheckpoint ( const dmtcp::vector<int>& fds, bool isRestart )
+{
+  JASSERT ( fds.size() > 0 );
+  JTRACE ( "RGARG, RGARG, RGARG, RGARG" ) ( fds[0] ) ( id() );
+}
+
+void dmtcp::EpollConnection::mergeWith ( const Connection& that)
+{
+  Connection::mergeWith(that);
+}
+
+void dmtcp::EpollConnection::restore(const dmtcp::vector<int>& fds,
+                                     ConnectionRewirer*)
+{
+  JASSERT (fds.size()>0);
+  JTRACE ( "RGARG, RGARG, RGARG, RGARG" ) ( fds[0] ) ( id() );
+  for(size_t i=0; i<fds.size(); ++i)
+  {
+    int fd = fds[i];
+    int tempFd =  _real_epoll_create ( _size ) ;
+    JASSERT ( tempFd >= 0 );
+    JWARNING ( _real_dup2 ( tempFd, fd ) == fd ) ( tempFd ) ( fd ) ( JASSERT_ERRNO );
+  }
+}
+
+void dmtcp::EpollConnection::restoreOptions ( const dmtcp::vector<int>& fds )
+{
+  Connection::restoreOptions ( fds );
+  JTRACE ( "RGARG, RGARG, RGARG, RGARG" ) ( fds[0] ) ( id() );
+  typedef dmtcp::map< int, struct epoll_event >::iterator fdEventIterator;
+  fdEventIterator fevt = _fdToEvent.begin();
+  for (; fevt != _fdToEvent.end(); fevt++)
+  {
+      JTRACE ("RGARG: restore sfd options") (fevt->first);
+      int ret = _real_epoll_ctl (fds[0], EPOLL_CTL_ADD, fevt->first, &(fevt->second));
+      if (ret)
+      {
+          JWARNING ("Error in restoring options") (ret) (strerror(errno));
+      }
+  }
+}
+
+void dmtcp::EpollConnection::serializeSubClass ( jalib::JBinarySerializer& o ){
+  JSERIALIZE_ASSERT_POINT ( "dmtcp::EpollConnection" );
+  o & _type & _stat;
+  o.serializeMap(_fdToEvent);
+  JTRACE ( "RGARG, RGARG, RGARG, RGARG" ) ;
+}
+
+dmtcp::EpollConnection& dmtcp::EpollConnection::asEpoll()
+{
+  JTRACE ( "RGARG, RGARG, RGARG, RGARG" ) ;
+  return *this;
+}
+
+void dmtcp::EpollConnection::onCTL (int op, int fd, struct epoll_event *event)
+{
+  if (really_verbose) {
+    JTRACE ("RGARG: EPOLL CTL-ing.") ( id() ) ( fd ) ( op );
+  }
+
+  JASSERT ( ((op==EPOLL_CTL_MOD || op==EPOLL_CTL_ADD) && event != NULL ) || op==EPOLL_CTL_DEL) ( epollType() ) ( id() )
+    .Text ( "Passing a NULL event! HUH!");
+
+  struct epoll_event myEvent;
+  _type = EPOLL_CTL;
+  if (op == EPOLL_CTL_DEL)
+  {
+      _fdToEvent.erase(fd);
+      return;
+  }
+  memcpy (&myEvent, event, sizeof myEvent);
+  _fdToEvent[fd] = myEvent;
+}
+
+void dmtcp::EventFdConnection::preCheckpoint ( const dmtcp::vector<int>& fds
+    , KernelBufferDrainer& drain )
+{
+  JASSERT ( fds.size() > 0 );
+
+  if (!hasLock(fds)) {
+    return;
+  }
+
+  _has_lock = true;
+
+  JTRACE ("Checkpoint eventfd.") (fds[0]);
+
+  int new_flags = (_fcntlFlags & (~(O_RDONLY|O_WRONLY))) | O_RDWR | O_NONBLOCK;
+  int evtfd = fds[0];
+  JASSERT(evtfd >= 0)(evtfd)(JASSERT_ERRNO);
+  // set the new flags
+  JASSERT (fcntl(evtfd, F_SETFL, new_flags) == 0) (evtfd) (new_flags) ( JASSERT_ERRNO );
+  ssize_t size;
+  uint64_t u;
+  unsigned int counter = 1;
+
+  // Read whatever is there on top of evtfd
+  size = read(evtfd, &u, sizeof(uint64_t));
+  if( -1 != size )
+  {
+      JTRACE ("Read value u: ") (evtfd) (u);
+      // EFD_SEMAPHORE flag not specified,
+      // the counter value would have been reset to 0 upon read
+      // Save the value, so that it can be restored in post-checkpoint
+      if (!(_flags & EFD_SEMAPHORE))
+      {
+        _initval = u;
+      }
+      else
+      {
+        // EFD_SEMAPHORE specified, so can't read the current counter value
+        // Keep reading till "semaphore" becomes 0.
+        while ( -1 != read(evtfd, &u, sizeof(uint64_t)) )
+          counter++;
+        _initval = counter;
+      }
+  }
+  else
+  {
+      JTRACE ("Nothing to be read from eventfd.") (evtfd) (errno) (strerror(errno));
+      _initval = 0;
+  }
+  //Connection::restoreOptions ( fds );
+  JTRACE ("Checkpointing eventfd:  end.") (fds[0]) (_initval);
+}
+
+void dmtcp::EventFdConnection::postCheckpoint ( const dmtcp::vector<int>& fds, bool isRestart )
+{
+  if( !_has_lock )
+    return; // nothing to do now
+  JTRACE ("Begin postCheckpoint eventfd.") (fds[0]);
+  JASSERT (fds.size() > 0);
+  uint64_t u = (unsigned long long) _initval;
+  for (unsigned int i=0; i<fds.size(); i++)
+  {
+      evtfd = fds[i];
+      if( -1 == write(evtfd, &u, sizeof(uint64_t)) )
+      {
+          JWARNING("Write to eventfd failed during postCheckpoint") (evtfd) (errno) (strerror(errno));
+      }
+  }
+  restoreOptions ( fds );
+  JTRACE ("End postCheckpoint eventfd.") (fds[0]);
+}
+
+void dmtcp::EventFdConnection::restoreOptions ( const dmtcp::vector<int>& fds )
+{
+  //call base version (F_GETFL etc)
+  Connection::restoreOptions ( fds );
+}
+
+
+void dmtcp::EventFdConnection::restore(const dmtcp::vector<int>& fds,
+                                       ConnectionRewirer*)
+{
+  JASSERT ( fds.size() > 0 );
+
+  JTRACE("Restoring EventFd Connection") (id());
+  errno = 0;
+  int tempfd = _real_eventfd (_initval, _flags);
+  JASSERT ( tempfd > 0 ) ( tempfd ) ( JASSERT_ERRNO );
+
+  for(size_t i=0; i<fds.size(); ++i)
+  {
+    JASSERT ( _real_dup2 ( tempfd, fds[i] ) == fds[i] ) ( tempfd ) ( fds[i] )
+      .Text ( "dup2() failed." );
+  }
+}
+
+void dmtcp::EventFdConnection::serializeSubClass ( jalib::JBinarySerializer& o )
+{
+  JSERIALIZE_ASSERT_POINT ( "dmtcp::EventFdConnection" );
+  o & _initval & _flags & _has_lock;
+  JTRACE("Serializing EvenFdConn.") ;
+}
+
+void dmtcp::SignalFdConnection::preCheckpoint ( const dmtcp::vector<int>& fds
+    , KernelBufferDrainer& drain )
+{
+  JASSERT ( fds.size() > 0 );
+
+  if (!hasLock(fds)) {
+    return;
+  }
+
+  _has_lock = true;
+
+  JTRACE ("Checkpoint signalfd.") (fds[0]);
+
+  int new_flags = (_fcntlFlags & (~(O_RDONLY|O_WRONLY))) | O_RDWR | O_NONBLOCK;
+  signlfd = fds[0];
+  JASSERT(signlfd >= 0)(signlfd)(JASSERT_ERRNO);
+  // set the new flags
+  JASSERT (fcntl(signlfd, F_SETFL, new_flags) == 0) (signlfd) (new_flags) ( JASSERT_ERRNO );
+  ssize_t size;
+  struct signalfd_siginfo fdsi;
+
+  // Read whatever is there on top of signalfd
+  size = read(signlfd, &fdsi, sizeof(struct signalfd_siginfo));
+  if( -1 != size )
+  {
+      // Save the value, so that it can be restored in post-checkpoint
+      memcpy(&_fdsi, &fdsi, sizeof (struct signalfd_siginfo));
+
+  }
+  else
+  {
+      JTRACE ("Nothing to be read from signalfd.") (signlfd) (errno) (strerror(errno));
+  }
+  JTRACE ("Checkpointing signlfd:  end.") (fds[0]) ;
+}
+
+void dmtcp::SignalFdConnection::postCheckpoint ( const dmtcp::vector<int>& fds, bool isRestart )
+{
+  if( !_has_lock )
+    return; // nothing to do now
+  JTRACE ("Begin postCheckpoint signalfd.") (fds[0]);
+  JASSERT (fds.size() > 0);
+  for (unsigned int i=0; i<fds.size(); i++)
+  {
+      //raise the signals
+      JTRACE ("Raising the signal...") (_fdsi.ssi_signo);
+      raise(_fdsi.ssi_signo);
+  }
+  restoreOptions ( fds );
+  JTRACE ("End postCheckpoint signalfd.") (fds[0]);
+}
+
+void dmtcp::SignalFdConnection::restoreOptions ( const dmtcp::vector<int>& fds )
+{
+  //call base version (F_GETFL etc)
+  Connection::restoreOptions ( fds );
+}
+
+
+void dmtcp::SignalFdConnection::restore(const dmtcp::vector<int>& fds,
+                                        ConnectionRewirer*)
+{
+  JASSERT ( fds.size() > 0 );
+
+  JTRACE("Restoring SignalFd Connection") (id());
+  errno = 0;
+  int tempfd = _real_signalfd (-1, &_mask, _flags);
+  JASSERT ( tempfd > 0 ) ( tempfd ) ( JASSERT_ERRNO );
+
+  for(size_t i=0; i<fds.size(); ++i)
+  {
+    JASSERT ( _real_dup2 ( tempfd, fds[i] ) == fds[i] ) ( tempfd ) ( fds[i] )
+      .Text ( "dup2() failed." );
+  }
+}
+
+void dmtcp::SignalFdConnection::serializeSubClass ( jalib::JBinarySerializer& o )
+{
+  JSERIALIZE_ASSERT_POINT ( "dmtcp::SignalFdConnection" );
+  o &  _flags & _mask & _fdsi & _has_lock;
+  JTRACE("Serializing SignalFdConn.") ;
 }
