@@ -1,6 +1,8 @@
 /****************************************************************************
- *   Copyright (C) 2006-2008 by Jason Ansel, Kapil Arya, and Gene Cooperman *
- *   jansel@csail.mit.edu, kapil@ccs.neu.edu, gene@ccs.neu.edu              *
+ *   Copyright (C) 2006-2008 by Jason Ansel, Kapil Arya, Gene Cooperman,    *
+ *                                                           and Rohan Garg *
+ *   jansel@csail.mit.edu, kapil@ccs.neu.edu, gene@ccs.neu.edu, and         *
+ *                                                      rohgarg@ccs.neu.edu *
  *                                                                          *
  *   This file is part of the dmtcp/src module of DMTCP (DMTCP:dmtcp/src).  *
  *                                                                          *
@@ -22,12 +24,14 @@
 #ifndef DMTCPCONNECTION_H
 #define DMTCPCONNECTION_H
 
+// THESE INCLUDES ARE IN RANDOM ORDER.  LET'S CLEAN IT UP AFTER RELEASE. - Gene
 #include "constants.h"
 #include "dmtcpalloc.h"
 #include "connectionidentifier.h"
 #include <vector>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <signal.h>
 #include <map>
 #include "../jalib/jbuffer.h"
 #include "../jalib/jserialize.h"
@@ -39,6 +43,34 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+#ifdef HAVE_EPOLL_H
+# include <sys/epoll.h>
+#else
+  /* KEEP THIS IN SYNC WITH syscallwrappers.h */
+# ifndef _SYS_EPOLL_H
+#  define _SYS_EPOLL_H    1
+   struct epoll_event {int dummy;};
+   /* Valid opcodes ( "op" parameter ) to issue to epoll_ctl().  */
+#  define EPOLL_CTL_ADD 1 /* Add a file decriptor to the interface.  */
+#  define EPOLL_CTL_DEL 2 /* Remove a file decriptor from the interface.  */
+#  define EPOLL_CTL_MOD 3 /* Change file decriptor epoll_event structure.  */
+# endif
+#endif
+#ifdef HAVE_EVENTFD_H
+# include <sys/eventfd.h>
+#else
+  enum { EFD_SEMAPHORE = 1 };
+#endif
+#ifdef HAVE_SIGNALFD_H
+# include <sys/signalfd.h>
+#else
+# include <stdint.h>
+  struct signalfd_siginfo {uint32_t ssi_signo; int dummy;};
+#endif
+
 namespace jalib { class JSocket; }
 
 namespace dmtcp
@@ -47,6 +79,7 @@ namespace dmtcp
   class KernelBufferDrainer;
   class ConnectionRewirer;
   class TcpConnection;
+  class EpollConnection;
   class KernelDeviceToConnection;
   class ConnectionToFds;
 
@@ -68,12 +101,16 @@ namespace dmtcp
         FILE    = 0x4000,
         STDIO   = 0x5000,
         FIFO    = 0x6000,
-        TYPEMASK = TCP | PIPE | PTY | FILE | STDIO | FIFO
+        EPOLL   = 0x7000,
+        EVENTFD = 0x8000,
+        SIGNALFD = 0x9000,
+        TYPEMASK = TCP | PIPE | PTY | FILE | STDIO | FIFO | EPOLL | EVENTFD | SIGNALFD
       };
 
       virtual ~Connection() {}
 
       int conType() const { return _type & TYPEMASK; }
+      int subType() const { return _type; }
       bool restoreInSecondIteration() { return _restoreInSecondIteration; }
 
       const ConnectionIdentifier& id() const { return _id; }
@@ -81,7 +118,8 @@ namespace dmtcp
       virtual void preCheckpoint ( const dmtcp::vector<int>& fds, KernelBufferDrainer& ) = 0;
       virtual void postCheckpoint ( const dmtcp::vector<int>& fds,
                                     bool isRestart = false) = 0;
-      virtual void restore ( const dmtcp::vector<int>&, ConnectionRewirer& ) = 0;
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL) = 0;
 
       virtual bool isDupConnection ( const Connection& _that,
                                      dmtcp::ConnectionToFds& conToFds ) { return false; };
@@ -97,6 +135,7 @@ namespace dmtcp
 
       //convert with type checking
       virtual TcpConnection& asTcp();
+      virtual EpollConnection& asEpoll();
 
       virtual void restartDup2(int oldFd, int newFd);
 
@@ -135,16 +174,17 @@ namespace dmtcp
 
       int tcpType() const { return _type; }
 
-#ifdef EXTERNAL_SOCKET_HANDLING
       enum PeerType
       {
         PEER_UNKNOWN,
         PEER_INTERNAL,
-        PEER_EXTERNAL
+        PEER_EXTERNAL,
+        PEER_SOCKETPAIR
       };
 
       enum PeerType peerType() const { return _peerType; }
 
+#ifdef EXTERNAL_SOCKET_HANDLING
       void markInternal() {
         if (_type == TCP_ACCEPT || _type == TCP_CONNECT)
           _peerType = PEER_INTERNAL;
@@ -159,6 +199,11 @@ namespace dmtcp
       // This accessor is needed because _type is protected.
       void markExternalConnect() { _type = TCP_EXTERNAL_CONNECT; }
 
+      void setSocketpairPeer(ConnectionIdentifier id) {
+        _peerType = PEER_SOCKETPAIR;
+        _socketpairPeerId = id;
+      }
+
       //basic commands for updating state from wrappers
       /*onSocket*/ TcpConnection ( int domain, int type, int protocol );
       void onBind ( const struct sockaddr* addr, socklen_t len );
@@ -177,7 +222,8 @@ namespace dmtcp
                                    , KernelBufferDrainer& drain );
       virtual void postCheckpoint ( const dmtcp::vector<int>& fds,
                                     bool isRestart = false);
-      virtual void restore ( const dmtcp::vector<int>&, ConnectionRewirer& );
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL);
 
       //virtual void doLocking ( const dmtcp::vector<int>& fds );
 
@@ -189,7 +235,11 @@ namespace dmtcp
       void sendHandshake(jalib::JSocket& sock, const dmtcp::UniquePid& coordinator);
       void recvHandshake(jalib::JSocket& sock, const dmtcp::UniquePid& coordinator);
 
+      void restoreSocketPair(const dmtcp::vector<int>& fds,
+                             dmtcp::TcpConnection *peer,
+                             const dmtcp::vector<int>& peerfds);
       const ConnectionIdentifier& getRemoteId() const { return _acceptRemoteId; }
+      const ConnectionIdentifier& getSocketpairPeerId() const { return _socketpairPeerId; }
 
       //called on restart when _id collides with another connection
       virtual void mergeWith ( const Connection& that );
@@ -201,9 +251,8 @@ namespace dmtcp
       int                     _sockType;
       int                     _sockProtocol;
       int                     _listenBacklog;
-#ifdef EXTERNAL_SOCKET_HANDLING
       enum PeerType           _peerType;
-#endif
+      bool                    _socketPairRestored;
       union {
         socklen_t               _bindAddrlen;
         socklen_t               _connectAddrlen;
@@ -214,6 +263,7 @@ namespace dmtcp
         struct sockaddr_storage _connectAddr;
       };
       ConnectionIdentifier    _acceptRemoteId;
+      ConnectionIdentifier    _socketpairPeerId;
       dmtcp::map< int, dmtcp::map< int, jalib::JBuffer > > _sockOptions; // _options[level][option] = value
   };
 
@@ -273,7 +323,8 @@ namespace dmtcp
                                    , KernelBufferDrainer& drain );
       virtual void postCheckpoint ( const dmtcp::vector<int>& fds,
                                     bool isRestart = false);
-      virtual void restore ( const dmtcp::vector<int>&, ConnectionRewirer& );
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL);
       virtual void restoreOptions ( const dmtcp::vector<int>& fds );
 
       virtual void serializeSubClass ( jalib::JBinarySerializer& o );
@@ -312,7 +363,8 @@ namespace dmtcp
                                    , KernelBufferDrainer& drain );
       virtual void postCheckpoint ( const dmtcp::vector<int>& fds,
                                     bool isRestart = false);
-      virtual void restore ( const dmtcp::vector<int>&, ConnectionRewirer& );
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL);
       virtual void restoreOptions ( const dmtcp::vector<int>& fds );
 
       virtual void serializeSubClass ( jalib::JBinarySerializer& o );
@@ -329,7 +381,8 @@ namespace dmtcp
       {
         FILE_INVALID = FILE,
         FILE_REGULAR,
-        FILE_DELETED
+        FILE_PROCFS,
+        FILE_DELETED,
       };
       //called on restart when _id collides with another connection
       virtual void mergeWith ( const Connection& that );
@@ -346,13 +399,15 @@ namespace dmtcp
         }
       }
 
+      virtual void doLocking ( const dmtcp::vector<int>& fds );
       virtual void preCheckpoint ( const dmtcp::vector<int>& fds
                                    , KernelBufferDrainer& drain );
       virtual void postCheckpoint ( const dmtcp::vector<int>& fds,
                                     bool isRestart = false);
 
       virtual void restoreOptions ( const dmtcp::vector<int>& fds );
-      virtual void restore ( const dmtcp::vector<int>&, ConnectionRewirer& );
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL);
 
       virtual void serializeSubClass ( jalib::JBinarySerializer& o );
 
@@ -366,7 +421,7 @@ namespace dmtcp
 
       bool isDupConnection ( const Connection& _that, dmtcp::ConnectionToFds& conToFds );
 
-      bool fileType() { return _type; }
+      int fileType() { return _type; }
 
     private:
       void saveFile (int fd);
@@ -414,7 +469,8 @@ namespace dmtcp
                                     bool isRestart = false);
 
       virtual void restoreOptions ( const dmtcp::vector<int>& fds );
-      virtual void restore ( const dmtcp::vector<int>&, ConnectionRewirer& );
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL);
 
       virtual void serializeSubClass ( jalib::JBinarySerializer& o );
 
@@ -432,6 +488,124 @@ namespace dmtcp
       vector<char> _in_data;
       int ckptfd;
   };
+
+  class EpollConnection: public Connection
+  {
+    public:
+      enum EpollType
+      {
+        EPOLL_INVALID = EPOLL,
+        EPOLL_CREATE,
+        EPOLL_CTL,
+        EPOLL_WAIT
+      };
+
+      inline EpollConnection (int size, int type=EPOLL_CREATE)
+          :Connection(EPOLL),
+           _type(type),
+           _size (size)
+      {
+        JTRACE ("new epoll connection created");
+      }
+
+      int epollType () const { return _type; }
+
+      virtual void preCheckpoint ( const dmtcp::vector<int>& fds, KernelBufferDrainer& );
+      virtual void postCheckpoint ( const dmtcp::vector<int>& fds,
+                                    bool isRestart = false);
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL);
+
+      //virtual void doLocking ( const dmtcp::vector<int>& fds );
+      //virtual void saveOptions ( const dmtcp::vector<int>& fds );
+      virtual void restoreOptions ( const dmtcp::vector<int>& fds );
+
+      //called on restart when _id collides with another connection
+      virtual void mergeWith ( const Connection& that );
+
+      //virtual void restartDup2(int oldFd, int newFd);
+      virtual void serializeSubClass ( jalib::JBinarySerializer& o );
+
+      virtual string str() { return "EPOLL-FD: <Not-a-File>"; };
+
+      void onCTL (int op, int fd, struct epoll_event *event);
+
+    private:
+      EpollConnection& asEpoll();
+      int         _type; // current state of EPOLL
+      struct stat _stat; // not sure if stat makes sense in case  of epfd
+      int         _size; // flags
+      dmtcp::map<int, struct epoll_event > _fdToEvent;
+  };
+
+  class EventFdConnection: public Connection
+  {
+    public:
+      inline EventFdConnection (unsigned int initval, int flags)
+          :Connection(EVENTFD),
+           _initval(initval),
+           _flags (flags)
+      {
+        JTRACE ("new eventfd connection created");
+      }
+
+      virtual void preCheckpoint ( const dmtcp::vector<int>& fds, KernelBufferDrainer& );
+      virtual void postCheckpoint ( const dmtcp::vector<int>& fds,
+                                    bool isRestart = false);
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL);
+
+      //virtual void doLocking ( const dmtcp::vector<int>& fds );
+      //virtual void saveOptions ( const dmtcp::vector<int>& fds );
+      virtual void restoreOptions ( const dmtcp::vector<int>& fds );
+
+      //virtual void restartDup2(int oldFd, int newFd);
+      virtual void serializeSubClass ( jalib::JBinarySerializer& o );
+
+      virtual string str() { return "EVENT-FD: <Not-a-File>"; };
+
+    private:
+      unsigned int   _initval; // initial counter value
+      int         _flags; // flags
+      bool _has_lock;
+      int evtfd;
+  };
+
+  class SignalFdConnection: public Connection
+  {
+    public:
+      inline SignalFdConnection (int signalfd, const sigset_t* mask, int flags)
+          :Connection(SIGNALFD),
+           signlfd (signalfd),
+           _flags (flags)
+      {
+        if (mask!=NULL)
+          _mask = *mask;
+        else
+          sigemptyset(&_mask);
+        memset(&_fdsi, 0, sizeof(_fdsi));
+        JTRACE ("new signalfd  connection created");
+      }
+
+      virtual void preCheckpoint ( const dmtcp::vector<int>& fds, KernelBufferDrainer& );
+      virtual void postCheckpoint ( const dmtcp::vector<int>& fds,
+                                    bool isRestart = false);
+      virtual void restore(const dmtcp::vector<int>&,
+                           ConnectionRewirer *rewirer = NULL);
+
+      virtual void restoreOptions ( const dmtcp::vector<int>& fds );
+
+      virtual void serializeSubClass ( jalib::JBinarySerializer& o );
+
+      virtual string str() { return "SIGNAL-FD: <Not-a-File>"; };
+
+    private:
+      int signlfd;
+      int         _flags; // flags
+      sigset_t _mask; // mask for signals
+      struct signalfd_siginfo _fdsi;
+      bool _has_lock;
+  };
 }
 
 #endif
