@@ -22,6 +22,7 @@
 #include "constants.h"
 #include "syscallwrappers.h"
 #include "connectionmanager.h"
+#include "dmtcpplugin.h"
 
 #include  "../jalib/jfilesystem.h"
 #include  "../jalib/jconvert.h"
@@ -43,9 +44,27 @@ static dmtcp::string _procFDPath ( int fd )
   return "/proc/self/fd/" + jalib::XToString ( fd );
 }
 
+static dmtcp::string _resolveSymlink(dmtcp::string path)
+{
+  dmtcp::string device = jalib::Filesystem::ResolveSymlink(path);
+  if (path.length() > 0 && dmtcp::Util::strStartsWith(device, "/proc/")) {
+    int index = 6;
+    char *rest;
+    char newpath[128];
+    JASSERT(device.length() < sizeof newpath);
+    pid_t realPid = strtol(&path[index], &rest, 0);
+    if (realPid > 0 && *rest == '/') {
+      pid_t virtualPid = CURRENT_TO_ORIGINAL_PID(realPid);
+      sprintf(newpath, "/proc/%d%s", virtualPid, rest);
+      device = newpath;
+    }
+  }
+  return device;
+}
+
 static bool _isBadFd ( int fd )
 {
-  dmtcp::string device = jalib::Filesystem::ResolveSymlink ( _procFDPath ( fd ) );
+  dmtcp::string device = _resolveSymlink( _procFDPath ( fd ) );
   return ( device == "" );
 }
 
@@ -158,7 +177,7 @@ dmtcp::string dmtcp::KernelDeviceToConnection::fdToDevice ( int fd, bool noOnDem
 {
   //gather evidence
   errno = 0;
-  dmtcp::string device = jalib::Filesystem::ResolveSymlink ( _procFDPath ( fd ) );
+  dmtcp::string device = _resolveSymlink(_procFDPath(fd));
   bool isBadFd = ( device == "" );
 
   if ( isBadFd )
@@ -169,18 +188,24 @@ dmtcp::string dmtcp::KernelDeviceToConnection::fdToDevice ( int fd, bool noOnDem
 
     return "";
   }
+  //while (fd==4);
 
   bool isFile  = ( device[0] == '/' );
 
   bool isTty = (device.compare("/dev/tty") == 0);
 
-  bool isPtmx  = (device.compare("/dev/ptmx") == 0);
-  bool isPts   = Util::strStartsWith(device, "/dev/pts/");
+  bool isPtmx  = (device.compare("/dev/ptmx") == 0 ||
+                  device.compare("/dev/pts/ptmx") == 0);
+
+  bool isPts   = !isPtmx && Util::strStartsWith(device, "/dev/pts/");
 
   bool isBSDMaster  = (Util::strStartsWith(device, "/dev/pty") &&
                        device.compare("/dev/pty") != 0);
   bool isBSDSlave   = (Util::strStartsWith(device, "/dev/tty") &&
                        device.compare("/dev/tty")) != 0;
+  bool isEpoll = (device.compare("anon_inode:[eventpoll]")==0);
+  bool isEventFd = (device.compare("anon_inode:[eventfd]")==0);
+  bool isSignalFd = (device.compare("anon_inode:[signalfd]")==0);
 #ifdef IBV
   bool isInfinibandDevice   = Util::strStartsWith(device, "/dev/infiniband/");
   bool isInfinibandConnection   = Util::strStartsWith(device, "infinibandevent:");
@@ -368,6 +393,57 @@ dmtcp::string dmtcp::KernelDeviceToConnection::fdToDevice ( int fd, bool noOnDem
       JASSERT(false) (device) .Text("Unimplemented file type.");
     }
   }
+  else if (isEventFd) {
+      dmtcp::string deviceName = "eventfd["+jalib::XToString(fd)+"]:"+device;
+      if (noOnDemandConnection)
+          return deviceName;
+      iterator i = _table.find(deviceName);
+      if (i == _table.end())
+      {
+          JTRACE("creating eventfd connection [on-demand]") (deviceName);
+          Connection *c = new EventFdConnection(0, 0);
+          ConnectionList::instance().add(c);
+          _table[deviceName] = c->id();
+          return deviceName;
+      }
+      else {
+          return deviceName;
+      }
+  }
+  else if (isSignalFd) {
+      dmtcp::string deviceName = "signalfd["+jalib::XToString(fd)+"]:"+device;
+      if (noOnDemandConnection)
+          return deviceName;
+      iterator i = _table.find(deviceName);
+      if (i == _table.end())
+      {
+          JTRACE("creating signalfd connection [on-demand]") (deviceName);
+          Connection *c = new SignalFdConnection(0, NULL, 0);
+          ConnectionList::instance().add(c);
+          _table[deviceName] = c->id();
+          return deviceName;
+      }
+      else {
+          return deviceName;
+      }
+  }
+  else if (isEpoll) {
+      dmtcp::string deviceName = "epoll["+jalib::XToString(fd)+"]:"+device;
+      if (noOnDemandConnection)
+          return deviceName;
+      iterator i = _table.find(deviceName);
+      if (i == _table.end())
+      {
+          JTRACE("creating eventfd connection [on-demand]") (deviceName);
+          Connection *c = new EpollConnection(5);
+          ConnectionList::instance().add(c);
+          _table[deviceName] = c->id();
+          return deviceName;
+      }
+      else {
+          return deviceName;
+      }
+  }
   //JWARNING(false) (device) .Text("Unimplemented Connection Type.");
   return device;
 }
@@ -469,8 +545,7 @@ dmtcp::KernelDeviceToConnection::KernelDeviceToConnection ( const ConnectionToFd
           JASSERT ( device == fdToDevice ( fds[i] ) )
             ( device ) ( fdToDevice ( fds[i] ) ) ( fds[i] ) ( fds[0] );
         } else {
-          dmtcp::string filePath =
-            jalib::Filesystem::ResolveSymlink ( _procFDPath ( fds[i] ) );
+          dmtcp::string filePath = _resolveSymlink(_procFDPath(fds[i]));
           JASSERT ( filePath == ((FileConnection *)c)->filePath() )
             ( fds[i] ) ( filePath ) ( fds[0] ) ( ((FileConnection *)c)->filePath() );
         }
@@ -540,6 +615,15 @@ void dmtcp::ConnectionList::serialize ( jalib::JBinarySerializer& o )
       case Connection::STDIO:
         con = new StdioConnection();
         break;
+      case Connection::EPOLL:
+        con = new EpollConnection(5); //dummy val
+        break;
+      case Connection::EVENTFD:
+        con = new EventFdConnection(0, 0); //dummy val
+        break;
+      case Connection::SIGNALFD:
+        con = new SignalFdConnection(0, NULL, 0); //dummy val
+        break;
       default:
         JASSERT ( false ) ( key ) ( o.filename() ).Text ( "unknown connection type" );
       }
@@ -741,6 +825,14 @@ dmtcp::Connection& dmtcp::ConnectionList::operator[] ( const ConnectionIdentifie
   return *_connections[id];
 }
 
+dmtcp::Connection *dmtcp::ConnectionList::getConnection(const ConnectionIdentifier& id)
+{
+  if (_connections.find(id) == _connections.end()) {
+    return NULL;
+  }
+  return _connections[id];
+}
+
 void dmtcp::ConnectionList::add ( Connection* c )
 {
   JWARNING ( _connections.find ( c->id() ) == _connections.end() ) ( c->id() )
@@ -796,7 +888,7 @@ bool dmtcp::SlidingFdTable::isInUse ( int fd ) const
   if ( _fdToCon.find ( fd ) != _fdToCon.end() )
     return true;
   //double check with the filesystem
-  dmtcp::string device = jalib::Filesystem::ResolveSymlink ( _procFDPath ( fd ) );
+  dmtcp::string device = _resolveSymlink(_procFDPath(fd));
   return device != "";
 }
 void dmtcp::SlidingFdTable::changeFd ( int oldfd, int newfd )
@@ -812,6 +904,11 @@ void dmtcp::SlidingFdTable::closeAll()
         ; i!=_conToFd.end()
         ; ++i )
   {
+    Connection& con = ConnectionList::instance() [i->first];
+    if (con.conType() == Connection::EPOLL) {
+      JTRACE("Delete epoll Connection");
+      //continue;
+    }
     JWARNING ( _real_close ( i->second ) ==0 ) ( i->second ) ( JASSERT_ERRNO );
   }
   _conToFd.clear();
