@@ -196,8 +196,7 @@ void dmtcp::ConnectionState::preCheckpointDrain()
     //we will create a new, broken socket that is not closed
 
     con.onError();
-    static ConnectionRewirer ignored;
-    con.restore(fds, ignored); //restoring a TCP_ERROR connection makes a dead socket
+    con.restore(fds); //restoring a TCP_ERROR connection makes a dead socket
     KernelDeviceToConnection::instance().redirect(fds[0], id);
   }
 
@@ -338,25 +337,47 @@ void dmtcp::ConnectionState::doReconnect ( jalib::JSocket& coordinator, jalib::J
   // Part 1: Restore all but Pseudo-terminal slaves and file connection which
   //         were not checkpointed
   ConnectionList::iterator i;
-  for ( i= connections.begin(); i != connections.end(); ++i )
-  {
-    JASSERT ( _conToFds[i->first].size() > 0 )
-      .Text ( "stale connections should be gone by now" );
+  for (i= connections.begin(); i != connections.end(); ++i) {
+    ConnectionIdentifier id = i->first;
+    Connection *con = i->second;
 
-    if ( (i->second)->restoreInSecondIteration() == false ) {
-      ( i->second )->restore ( _conToFds[i->first], _rewirer );
+    JASSERT(_conToFds[id].size() > 0)
+      .Text("stale connections should be gone by now");
+
+    if (con->subType() == FileConnection::FILE_PROCFS) {
+      continue;
+    }
+
+    if (con->conType() == Connection::TCP) {
+      TcpConnection *tcpCon = (TcpConnection *) con;
+      if (tcpCon->peerType() == TcpConnection::PEER_SOCKETPAIR) {
+        ConnectionIdentifier peerId = tcpCon->getSocketpairPeerId();
+        TcpConnection *peerCon = (TcpConnection*) connections.getConnection(peerId);
+        if (peerCon != NULL) {
+          tcpCon->restoreSocketPair(_conToFds[id], peerCon, _conToFds[peerId]);
+          continue;
+        }
+      }
+    }
+
+    if (con->restoreInSecondIteration() == false) {
+      con->restore(_conToFds[id], &_rewirer);
     }
   }
 
   // Part 2: Restore all Pseudo-terminal slaves and file connections that were
   //         not checkpointed.
-  for ( i= connections.begin(); i != connections.end(); ++i )
-  {
-    JASSERT ( _conToFds[i->first].size() > 0 )
-      .Text ( "stale connections should be gone by now" );
+  for (i = connections.begin(); i != connections.end(); ++i) {
+    Connection *con = i->second;
+    JASSERT(_conToFds[i->first].size() > 0)
+      .Text("stale connections should be gone by now");
 
-    if ( ( i->second )->restoreInSecondIteration() == true ) {
-      ( i->second )->restore ( _conToFds[i->first], _rewirer );
+    if (con->subType() == FileConnection::FILE_PROCFS) {
+      continue;
+    }
+
+    if (con->restoreInSecondIteration() == true) {
+      con->restore(_conToFds[i->first], &_rewirer);
     }
   }
   _rewirer.doReconnect();
@@ -372,6 +393,8 @@ void dmtcp::ConnectionState::handleDuplicateFilesInSeparateConnections()
   for ( i= connections.begin(); i != connections.end(); ++i )
   {
     if ( i->second->conType() != Connection::FILE ) continue;
+    if ( i->second->conType() == Connection::EPOLL)
+        out << "found epoll fd\n";
 
     FileConnection* fileConI = (FileConnection*) i->second;
     if (fileConI->checkpointed() == false) continue;
@@ -391,5 +414,4 @@ void dmtcp::ConnectionState::handleDuplicateFilesInSeparateConnections()
       }
     }
   }
-  JTRACE ("Duplicate files in separate connections") (out.str());
-}
+  JTRACE ("Duplicate files in separate connections") (out.str()); }
