@@ -87,6 +87,8 @@ static const char* theUsage =
   "        a randomly assigned port (if no port is specified by --port)\n"
   "  --interval, -i, (environment variable DMTCP_CHECKPOINT_INTERVAL):\n"
   "      Time in seconds between automatic checkpoints.\n"
+  "      0 implies never (manual ckpt only); if not set and no env var,\n"
+  "        use default value set in dmtcp_coordinator or dmtcp_command.\n"
   "      Not allowed if --join is specified\n"
   "      --batch implies -i 3600, unless otherwise specified.\n"
   "  --no-check:\n"
@@ -95,8 +97,8 @@ static const char* theUsage =
   "      Checkpoint open files and restore old working dir. (Default: do neither)\n"
   "  --mtcp-checkpoint-signal:\n"
   "      Signal number used internally by MTCP for checkpointing (default: 12)\n"
-  "  --with-module (environment variable DMTCP_MODULE):\n"
-  "      Colon-separated list of DMTCP modules to be preloaded with DMTCP.\n"
+  "  --with-plugin (environment variable DMTCP_PLUGIN):\n"
+  "      Colon-separated list of DMTCP plugins to be preloaded with DMTCP.\n"
   "      (Absolute pathnames are required.)\n"
   "  --quiet, -q, (or set environment variable DMTCP_QUIET = 0, 1, or 2):\n"
   "      Skip banner and NOTE messages; if given twice, also skip WARNINGs\n"
@@ -215,8 +217,11 @@ static void processArgs(int *orig_argc, char ***orig_argv)
     } else if (s == "--checkpoint-open-files") {
       checkpointOpenFiles = true;
       shift;
-    } else if (s == "--with-module") {
-      setenv(ENV_VAR_MODULE, argv[1], 1);
+    } else if (s == "--with-plugin") {
+      setenv(ENV_VAR_PLUGIN, argv[1], 1);
+      shift; shift;
+    } else if (s == "--with-module") { /* TEMPORARILY SUPPORT BACKWARD COMPAT.*/
+      setenv(ENV_VAR_PLUGIN, argv[1], 1);
       shift; shift;
     } else if (s == "-q" || s == "--quiet") {
       *getenv(ENV_VAR_QUIET) = *getenv(ENV_VAR_QUIET) + 1;
@@ -412,13 +417,13 @@ int main ( int argc, char** argv )
      dmtcp::DmtcpCoordinatorAPI::startCoordinatorIfNeeded(allowedModes);
 
   // preloadLibs are to set LD_PRELOAD:
-  //   LD_PRELOAD=MODULE_LIBS:UTILITY_DIR/dmtcphijack.so:R_LIBSR_UTILITY_DIR/
+  //   LD_PRELOAD=PLUGIN_LIBS:UTILITY_DIR/dmtcphijack.so:R_LIBSR_UTILITY_DIR/
   dmtcp::string preloadLibs = "";
-  // FIXME:  If the colon-separated elements of ENV_VAR_MODULE are not
+  // FIXME:  If the colon-separated elements of ENV_VAR_PLUGIN are not
   //     absolute pathnames, then they must be expanded to absolute pathnames.
   //     Warn user if an absolute pathname is not valid.
-  if ( getenv(ENV_VAR_MODULE) != NULL ) {
-    preloadLibs += getenv(ENV_VAR_MODULE);
+  if ( getenv(ENV_VAR_PLUGIN) != NULL ) {
+    preloadLibs += getenv(ENV_VAR_PLUGIN);
     preloadLibs += ":";
   }
   // FindHelperUtiltiy requires ENV_VAR_UTILITY_DIR to be set
@@ -541,7 +546,8 @@ void testStaticallyLinked(const char *pathname) {
   if (dmtcp::Util::isStaticallyLinked(pathname)) {
     JASSERT_STDERR <<
       "*** WARNING:  /lib/ld-linux.so --verify " << pathname << " returns\n"
-      << "***  nonzero status.  (Some distros use /lib64/ld-linux-x86-64.so .)"
+      << "***  nonzero status.  (Some distros"
+         " use /lib64/ld-linux-x86-64.so .)\n"
       << "*** This often means that " << pathname << " is\n"
       << "*** a statically linked target.  If so, you can confirm this with\n"
       << "*** the 'file' command.\n"
