@@ -446,7 +446,6 @@ namespace dmtcp
       Util::initializeLogFile(procname());
       JTRACE("Creating process during restart") (upid()) (procname());
 
-
       VirtualPidTable &vt = _virtualPidTable;
 
       JTRACE("")(_real_getpid())(_real_getppid())(_real_getsid(0));
@@ -454,6 +453,23 @@ namespace dmtcp
       vt.updateMapping(upid().pid(), _real_getpid());
       pid_t psid = vt.sid();
 
+      JTRACE("Restore /proc/self/* fds");
+      ConnectionList& connections = ConnectionList::instance();
+      ConnectionList::iterator it;
+      for (it = connections.begin(); it != connections.end(); ++it) {
+        dmtcp::Connection *con = it->second;
+        if (con->subType() == FileConnection::FILE_PROCFS) {
+          dmtcp::FileConnection *filecon = (dmtcp::FileConnection*) con;
+          char buf[32];
+          dmtcp::vector<int> fds;
+          fds.push_back(slidingFd.getFdFor(con->id()));
+          sprintf(buf, "/proc/%d/", vt.pid());
+          if (dmtcp::Util::strStartsWith(filecon->filePath(), buf)) {
+            filecon->restore(fds);
+          }
+        }
+      }
+
       if( !isSessionLeader() ){
 
 	// Restore Group information
@@ -544,8 +560,9 @@ namespace dmtcp
 	}
       }
 
+      bool isTheGroupLeader = isGroupLeader(); // Calls JTRACE, avoid recursion
       JTRACE("Child and dependent root processes forked, restoring process")
-	    (upid())(getpid())(isGroupLeader());
+	    (upid())(getpid())(isTheGroupLeader);
       // Save PID mapping information
       pid_t orig = upid().pid();
       pid_t curr = _real_getpid();
@@ -658,6 +675,8 @@ static const char* theUsage =
   "        a randomly assigned port (if no port is specified by --port)\n"
   "  --interval, -i, (environment variable DMTCP_CHECKPOINT_INTERVAL):\n"
   "      Time in seconds between automatic checkpoints.\n"
+  "      0 implies never (manual ckpt only); if not set and no env var,\n"
+  "        use default value set in dmtcp_coordinator or dmtcp_command.\n"
   "      Not allowed if --join is specified\n"
   "      --batch implies -i 3600, unless otherwise specified.\n"
   "  --no-check:\n"
@@ -856,7 +875,7 @@ int main ( int argc, char** argv )
       ( targets[i]._path ) ( targets[i]._compGroup ) ( targets[i]._numPeers );
     if( compGroup != targets[i]._compGroup){
       JASSERT(false)(compGroup)(targets[i]._compGroup)
-	.Text("ERROR: Restored programs belongs to different computation IDs");
+	.Text("ERROR: Restored programs belong to different computation IDs");
     }else if( numPeers != targets[i]._numPeers ){
       JASSERT(false)(numPeers)(targets[i]._numPeers)
 	.Text("ERROR: Different number of processes saved in checkpoint images");
