@@ -164,8 +164,13 @@ void dmtcp::DmtcpCoordinatorAPI::sendCoordinatorHandshake (
   hello_local.restorePort = theRestorePort;
 
   const char* interval = getenv ( ENV_VAR_CKPT_INTR );
+  /* DmtcpMessage constructor default:
+   *   hello_local.theCheckpointInterval: DMTCPMESSAGE_SAME_CKPT_INTERVAL
+   */
   if ( interval != NULL )
     hello_local.theCheckpointInterval = jalib::StringToInt ( interval );
+  // Tell the coordinator the ckpt interval only once.  It can change later.
+  _dmtcp_unsetenv ( ENV_VAR_CKPT_INTR );
 
   hello_local.extraBytes = hostname.length() + 1 + progname.length() + 1;
 
@@ -316,8 +321,10 @@ void dmtcp::DmtcpCoordinatorAPI::startCoordinatorIfNeeded(int modes,
     if(WEXITSTATUS(coordinatorStatus) == CS_NO){
       JASSERT (false) (isRestart)
 	 .Text ("Coordinator in a funny state.  Peers exist, not restarting," \
-		"\n but not in a running state.  Checkpointing?" \
-		"\n Or maybe restarting and running with peers existing?");
+		"\n but not in a running state.  Often this means you are" \
+		" connecting to\n a stale coordinator from a previous" \
+		" computation.\n Try killing the other coordinator," \
+		" or using a different port for the new comp.");
     }else if (WEXITSTATUS(coordinatorStatus) == DMTCP_FAIL_RC) {
       JTRACE("Coordinator not found.  Starting a new one.");
     }else{
