@@ -25,7 +25,7 @@
 #include  "../jalib/jconvert.h"
 #include  "../jalib/jalloc.h"
 #include "dmtcpmessagetypes.h"
-#include "dmtcpmodule.h"
+#include "dmtcpplugin.h"
 #include <stdlib.h>
 #include "mtcpinterface.h"
 #include <unistd.h>
@@ -177,7 +177,7 @@ extern "C" LIB_PRIVATE void prepareDmtcpWrappers()
    * child process. This needs to be the first function that is called by
    * libc:fork() after the child process is created.
    *
-   * Some dmtcp module might also call pthread_atfork and so we call it right
+   * Some dmtcp plugin might also call pthread_atfork and so we call it right
    * here before initializing the wrappers.
    *
    * NOTE: If this doesn't work and someone is able to call pthead_atfork
@@ -352,7 +352,7 @@ dmtcp::DmtcpWorker::DmtcpWorker ( bool enableCheckpointing )
 
   connectToCoordinatorWithHandshake();
 
-  // define "Weak Symbols for each library module in dmtcphijack.so
+  // define "Weak Symbols for each library plugin in dmtcphijack.so
   dmtcp_process_event(DMTCP_EVENT_INIT, NULL);
 
   /* Acquire the lock here, so that the checkpoint-thread won't be able to
@@ -737,7 +737,7 @@ void dmtcp::DmtcpWorker::waitForStage1Suspend()
 #endif
 
   waitForCoordinatorMsg ( "SUSPEND", DMT_DO_SUSPEND );
-  UniquePid::updateCheckpointDirName();
+  UniquePid::updateCkptDir();
 
   JTRACE("got SUSPEND message, preparing to acquire all Thread-sync locks");
   ThreadSync::acquireLocks();
@@ -774,11 +774,7 @@ void dmtcp::DmtcpWorker::waitForStage2Checkpoint()
   JTRACE ( "locked" );
 
 #ifdef PID_VIRTUALIZATION
-  /*
-   * Save first 2 * sizeof(pid_t) bytes of each shared memory area and fill it
-   * with all zeros.
-   */
-  SysVIPC::instance().prepareForLeaderElection();
+  SysVIPC::instance().leaderElection();
 #endif
 
   WorkerState::setCurrentState ( WorkerState::FD_LEADER_ELECTION );
@@ -798,14 +794,7 @@ void dmtcp::DmtcpWorker::waitForStage2Checkpoint()
   JTRACE ( "drained" );
 
 #ifdef PID_VIRTUALIZATION
-  /*
-   * write pid at offset 0. Also write pid at offset sizeof(pid_t) if this
-   * process is the creator of this memory area. After the leader election
-   * barrier, the leader of the shared-memory object is the creator of the
-   * object. If the creator process is missing, then the leader process is the
-   * process whose pid is stored at offset 0
-   */
-  SysVIPC::instance().leaderElection();
+  SysVIPC::instance().preCkptDrain();
 #endif
 
   WorkerState::setCurrentState ( WorkerState::DRAINED );
@@ -940,7 +929,7 @@ void dmtcp::DmtcpWorker::writeCheckpointPrefix ( int fd )
 void dmtcp::DmtcpWorker::sendCkptFilenameToCoordinator()
 {
   // Tell coordinator to record our filename in the restart script
-  dmtcp::string ckptFilename = dmtcp::UniquePid::checkpointFilename();
+  dmtcp::string ckptFilename = dmtcp::UniquePid::getCkptFilename();
   dmtcp::string hostname = jalib::Filesystem::GetCurrentHostname();
   JTRACE ( "recording filenames" ) ( ckptFilename ) ( hostname );
   dmtcp::DmtcpMessage msg;
