@@ -73,7 +73,7 @@ LIB_PRIVATE void pthread_atfork_prepare()
    * cause several issues. One solution to this problem is to call the fork
    * system call directly whenever a tid-conflict is detected, however, it
    * might have some other side-effects.  Another possible solution would be to
-   * have pid-virtualization module, which always assigns virtual pids, to the
+   * have pid-virtualization plugin, which always assigns virtual pids, to the
    * newly created processes, and thus avoiding the pid-conflict totally.
    */
   return;
@@ -200,12 +200,12 @@ static void execShortLivedProcessAndExit(const char *path, char *const argv[])
   memset(buf, 0, bufSize);
   FILE *output;
   if (argv[0] == NULL) {
-    output = popen(path, "r");
+    output = _real_popen(path, "r");
   } else {
     dmtcp::string command = path;
     for (int i = 1; argv[i] != NULL; i++)
       command = command + " " + argv[i];
-    output = popen(command.c_str(), "r");
+    output = _real_popen(command.c_str(), "r");
   }
   int numRead = fread(buf, 1, bufSize, output);
   numRead++, numRead--; // suppress unused-var warning
