@@ -19,6 +19,12 @@
  *  <http://www.gnu.org/licenses/>.                                         *
  ****************************************************************************/
 
+/* ptsname_r is declared with "always_inline" attribute. GCC 4.7+ disallows us
+ * to define the ptsname_r wrapper if compiled with -O0. Thus we are disabling
+ * that "always_inline" definition here.
+*/
+#define ptsname_r ptsname_r_always_inline
+
 #include <stdarg.h>
 #include <stdlib.h>
 #include <vector>
@@ -44,9 +50,12 @@
 #include "syscallwrappers.h"
 #include "sysvipc.h"
 #include "util.h"
+#include "dmtcpplugin.h"
 #include  "../jalib/jassert.h"
 #include  "../jalib/jconvert.h"
 
+#undef ptsname_r
+extern "C" int ptsname_r ( int fd, char * buf, size_t buflen );
 
 #ifdef EXTERNAL_SOCKET_HANDLING
 extern dmtcp::vector <dmtcp::ConnectionIdentifier> externalTcpConnections;
@@ -176,10 +185,23 @@ extern "C" int ptsname_r ( int fd, char * buf, size_t buflen )
   return retVal;
 }
 
+extern "C" int __ptsname_r_chk (int fd, char * buf, size_t buflen, size_t nreal)
+{
+  WRAPPER_EXECUTION_DISABLE_CKPT();
+
+  JASSERT (buflen <= nreal) (buflen) (nreal) .Text("Buffer Overflow detected!");
+
+  int retVal = ptsname_r_work(fd, buf, buflen);
+
+  WRAPPER_EXECUTION_ENABLE_CKPT();
+
+  return retVal;
+}
+
 #ifdef PID_VIRTUALIZATION
 #include <virtualpidtable.h>
 
-static void updateProcPath ( const char *path, char *newpath )
+static void updateProcPathOriginalToCurrent ( const char *path, char *newpath )
 {
   char temp [ 10 ];
   int index, tempIndex;
@@ -215,8 +237,41 @@ static void updateProcPath ( const char *path, char *newpath )
   else strcpy ( newpath, path );
   return;
 }
+
+static void updateProcPathCurrentToOriginal(const char *path, char *newpath)
+{
+  if (path == NULL || strlen(path) == 0) {
+    strcpy(newpath, "");
+    return;
+  }
+
+  if (dmtcp::Util::strStartsWith(path, "/proc/")) {
+    int index = 6;
+    char *rest;
+    pid_t realPid = strtol(&path[index], &rest, 0);
+    if (realPid > 0 && *rest == '/') {
+      pid_t originalPid = CURRENT_TO_ORIGINAL_PID(realPid);
+      sprintf(newpath, "/proc/%d%s", originalPid, rest);
+    } else {
+      strcpy(newpath, path);
+    }
+  } else {
+    strcpy(newpath, path);
+  }
+  return;
+}
+
 #else
-void updateProcPath ( const char *path, char *newpath )
+void updateProcPathOriginalToCurrent ( const char *path, char *newpath )
+{
+  if (  path == "" || path == NULL ) {
+    strcpy( newpath, "" );
+    return;
+  }
+  strcpy ( newpath, path );
+  return;
+}
+void updateProcPathCurrentToOriginal ( const char *path, char *newpath )
 {
   if (  path == "" || path == NULL ) {
     strcpy( newpath, "" );
@@ -277,7 +332,9 @@ static void processDevPtmxConnection (int fd)
   uniquePtsNameStr = UNIQUE_PTS_PREFIX_STR;
   uniquePtsNameStr += jalib::XToString(getNextFreeSlavePtyNum());
 
-  dmtcp::string deviceName = "ptmx[" + ptsNameStr + "]:" + "/dev/ptmx";
+  dmtcp::string ptmxDevice =
+    jalib::Filesystem::ResolveSymlink("/proc/self/fd/" + jalib::XToString(fd));
+  dmtcp::string deviceName = "ptmx[" + ptsNameStr + "]:" + ptmxDevice; //"/dev/ptmx";
 
 //   dmtcp::string deviceName = "ptmx[" + dmtcp::UniquePid::ThisProcess().toString()
 //                            + ":" + jalib::XToString ( _nextPtmxId() )
@@ -329,7 +386,7 @@ static int _open_open64_work(int (*fn)(const char *path, int flags, ...),
       dmtcp::UniquePtsNameToPtmxConId::instance().retrieveCurrentPtsDeviceName(path);
     strcpy(newpath, currPtsDevName.c_str());
   } else {
-    updateProcPath ( path, newpath );
+    updateProcPathOriginalToCurrent ( path, newpath );
   }
 
   int fd = (*fn)( newpath, flags, mode );
@@ -394,7 +451,7 @@ static FILE *_fopen_fopen64_work(FILE* (*fn)(const char *path, const char *mode)
       dmtcp::UniquePtsNameToPtmxConId::instance().retrieveCurrentPtsDeviceName(path);
     strcpy(newpath, currPtsDevName.c_str());
   } else {
-    updateProcPath ( path, newpath );
+    updateProcPathOriginalToCurrent ( path, newpath );
   }
 
   FILE *file = (*fn) ( newpath, mode );
@@ -432,7 +489,7 @@ static void updateStatPath(const char *path, char *newpath)
     dmtcp::string currPtsDevName = dmtcp::UniquePtsNameToPtmxConId::instance().retrieveCurrentPtsDeviceName(path);
     strcpy(newpath, currPtsDevName.c_str());
   } else {
-    updateProcPath ( path, newpath );
+    updateProcPathOriginalToCurrent ( path, newpath );
   }
 }
 
@@ -499,13 +556,75 @@ extern "C" READLINK_RET_TYPE readlink(const char *path, char *buf,
 {
   char newpath [ PATH_MAX ] = {0} ;
   WRAPPER_EXECUTION_DISABLE_CKPT();
-  updateProcPath(path, newpath);
-  READLINK_RET_TYPE retval;
-  retval = _real_readlink(newpath, buf, bufsiz);
+  updateProcPathOriginalToCurrent(path, newpath);
+  READLINK_RET_TYPE retval = _real_readlink(newpath, buf, bufsiz);
+
+#if 0
+  if (retval != -1) {
+    updateProcPathCurrentToOriginal(buf, newpath);
+    JASSERT(strlen(newpath) < bufsiz);
+    strcpy(buf, newpath);
+  }
+#endif
   WRAPPER_EXECUTION_ENABLE_CKPT();
   return retval;
 }
 
+extern "C" char *realpath(const char *path, char *resolved_path)
+{
+  char newpath [ PATH_MAX ] = {0} ;
+  updateProcPathOriginalToCurrent(path, newpath);
+  char *retval = NEXT_FNC(realpath) (newpath, resolved_path);
+  if (retval != NULL) {
+    updateProcPathCurrentToOriginal(retval, newpath);
+    strcpy(retval, newpath);
+  }
+  return retval;
+}
+
+extern "C" char *__realpath(const char *path, char *resolved_path)
+{
+  char newpath [ PATH_MAX ] = {0} ;
+  updateProcPathOriginalToCurrent(path, newpath);
+  char *retval = NEXT_FNC(__realpath) (newpath, resolved_path);
+  if (retval != NULL) {
+    updateProcPathCurrentToOriginal(retval, newpath);
+    strcpy(retval, newpath);
+  }
+  return retval;
+}
+extern "C" char *__realpath_chk(const char *path, char *resolved_path,
+                                size_t resolved_len)
+{
+  char newpath [ PATH_MAX ] = {0} ;
+  updateProcPathOriginalToCurrent(path, newpath);
+  char *retval = NEXT_FNC(__realpath_chk) (newpath, resolved_path, resolved_len);
+  if (retval != NULL) {
+    updateProcPathCurrentToOriginal(retval, newpath);
+    JASSERT(strlen(newpath) < resolved_len);
+    strcpy(resolved_path, newpath);
+  }
+  return retval;
+}
+
+extern "C" char *canonicalize_file_name(const char *path)
+{
+  char newpath [ PATH_MAX ] = {0} ;
+  updateProcPathOriginalToCurrent(path, newpath);
+  char *retval = NEXT_FNC(canonicalize_file_name) (newpath);
+  if (retval != NULL) {
+    updateProcPathCurrentToOriginal(retval, newpath);
+    strcpy(retval, newpath);
+  }
+  return retval;
+}
+
+extern "C" int access(const char *path, int mode)
+{
+  char newpath [ PATH_MAX ] = {0} ;
+  updateProcPathOriginalToCurrent(path, newpath);
+  return NEXT_FNC(access) (newpath, mode);
+}
 
 #ifdef PID_VIRTUALIZATION
 // TODO:  ioctl must use virtualized pids for request = TIOCGPGRP / TIOCSPGRP
