@@ -73,6 +73,8 @@ extern "C" int socketpair ( int d, int type, int protocol, int sv[2] )
   a = new dmtcp::TcpConnection ( d, type, protocol );
   a->onConnect();
   b = new dmtcp::TcpConnection ( *a, a->id() );
+  a->setSocketpairPeer(b->id());
+  b->setSocketpairPeer(a->id());
 
   dmtcp::KernelDeviceToConnection::instance().create ( sv[0] , a );
   dmtcp::KernelDeviceToConnection::instance().create ( sv[1] , b );
@@ -568,6 +570,12 @@ extern "C" long int syscall(long int sys_num, ... )
     }
 # endif
 #endif
+    case SYS_poll:
+    {
+      SYSCALL_GET_ARGS_3(struct pollfd *,fds,nfds_t,nfds,int,timeout);
+      ret = poll(fds, nfds, timeout);
+      break;
+    }
     case SYS_epoll_create:
     {
       SYSCALL_GET_ARG(int,size);
