@@ -37,20 +37,13 @@
 #include "virtualpidtable.h"
 #include "protectedfds.h"
 #include "sockettable.h"
-#include "dmtcpmodule.h"
+#include "dmtcpplugin.h"
 
 #include "../jalib/jfilesystem.h"
 #include "../jalib/jconvert.h"
 #include "../jalib/jassert.h"
 #include "../jalib/jalloc.h"
 
-#ifdef __x86_64__
-# define MTCP_RESTORE_STACK_BASE ((char*)0x7FFFFFFFF000L)
-#else
-# define MTCP_RESTORE_STACK_BASE \
-    (strcmp("#CONFIG_M32","yes") == 0 ? ((char *)NULL) : ((char*)0xC0000000L))
-#endif
-
 #ifdef DEBUG
 static int debugEnabled = 1;
 #else
@@ -93,8 +86,8 @@ LIB_PRIVATE MtcpFuncPtrs_t mtcpFuncPtrs;
 static bool delayedCheckpoint = false;
 #endif
 
-//to allow linking without ptrace module
-int __attribute__ ((weak)) mtcp_is_ptracing() { return FALSE; }
+//to allow linking without ptrace plugin
+extern "C" int __attribute__ ((weak)) mtcp_is_ptracing() { return FALSE; }
 
 static void* find_and_open_mtcp_so()
 {
@@ -212,10 +205,10 @@ void dmtcp::initializeMtcpEngine()
                                      );
 
   JTRACE ("Calling mtcp_init");
-  mtcpFuncPtrs.init(UniquePid::checkpointFilename(), 0xBadF00d, 1);
+  mtcpFuncPtrs.init(UniquePid::getCkptFilename(), 0xBadF00d, 1);
   mtcpFuncPtrs.ok();
 
-  JTRACE ( "mtcp_init complete" ) ( UniquePid::checkpointFilename() );
+  JTRACE ( "mtcp_init complete" ) ( UniquePid::getCkptFilename() );
 }
 
 static void callbackSleepBetweenCheckpoint ( int sec )
@@ -255,14 +248,7 @@ static void callbackPreCheckpoint( char ** ckptFilename )
 #else
   dmtcp::DmtcpWorker::instance().waitForStage2Checkpoint();
 #endif
-  {
-    // If we don't modify *ckptFilename, then MTCP will continue to use
-    //  its default filename, which was passed to it via our call to mtcp_init()
-#ifdef UNIQUE_CHECKPOINT_FILENAMES
-    //dmtcp::UniquePid::ThisProcess().incrementGeneration();
-    *ckptFilename = const_cast<char *>(dmtcp::UniquePid::checkpointFilename());
-#endif
-  }
+  *ckptFilename = const_cast<char *>(dmtcp::UniquePid::getCkptFilename());
   JTRACE ( "MTCP is about to write checkpoint image." )(*ckptFilename);
 }
 
@@ -359,7 +345,7 @@ static void callbackRestoreVirtualPidTable()
 
 void callbackHoldsAnyLocks(int *retval)
 {
-  /* This callback is useful only for the ptrace module currently, but may be
+  /* This callback is useful only for the ptrace plugin currently, but may be
    * used for other stuff as well.
    *
    * This is invoked as the first thing in stopthisthread() routine, which is
@@ -467,11 +453,9 @@ static void restoreArgvAfterRestart(char* mtcpRestoreArgvStartAddr)
   long page_size = sysconf(_SC_PAGESIZE);
   long page_mask = ~(page_size - 1);
   char *startAddr = (char*) ((unsigned long) mtcpRestoreArgvStartAddr & page_mask);
-  char *endAddr = MTCP_RESTORE_STACK_BASE;
-  size_t len = endAddr - startAddr;
-#ifdef CONFIG_M32
-  return;
-#endif
+
+  size_t len;
+  len = (dmtcp::DmtcpWorker::argvSize() + page_size) & page_mask;
 
   // Check to verify if any page in the given range is already mmap()'d.
   // It assumes that the given addresses may belong to stack only and if
@@ -496,7 +480,7 @@ static void restoreArgvAfterRestart(char* mtcpRestoreArgvStartAddr)
     // Do NOT change restarted process's /proc/self/cmdline.
     //args[0] = DMTCP_PRGNAME_PREFIX + args[0];
     for ( size_t i=0; i< args.size(); ++i ) {
-      if (addr + args[i].length() >= endAddr)
+      if (addr + args[i].length() >= startAddr + len)
         break;
       strcpy(addr, args[i].c_str());
       addr += args[i].length() + 1;
@@ -511,10 +495,12 @@ static void restoreArgvAfterRestart(char* mtcpRestoreArgvStartAddr)
 
 static void unmapRestoreArgv()
 {
+  long page_size = sysconf(_SC_PAGESIZE);
+  long page_mask = ~(page_size - 1);
   if (_mtcpRestoreArgvStartAddr != NULL) {
     JTRACE("Unmapping previously mmap()'d pages (that were mmap()'d for restoring argv");
-    char *endAddr = MTCP_RESTORE_STACK_BASE;
-    size_t len = endAddr - _mtcpRestoreArgvStartAddr;
+    size_t len;
+    len = (dmtcp::DmtcpWorker::argvSize() + page_size) & page_mask;
     JASSERT(_real_munmap(_mtcpRestoreArgvStartAddr, len) == 0)
       (_mtcpRestoreArgvStartAddr) (len)
       .Text ("Failed to munmap extra pages that were mapped during restart");
