@@ -408,6 +408,10 @@ int _real_shmctl (int shmid, int cmd, struct shmid_ds *buf) {
   REAL_FUNC_PASSTHROUGH ( shmctl ) (shmid, cmd, buf);
 }
 
+LIB_PRIVATE
+int _real_poll(struct pollfd *fds, nfds_t nfds, int timeout) {
+  REAL_FUNC_PASSTHROUGH (poll) (fds, nfds, timeout);
+}
 
 LIB_PRIVATE
 int _real_epoll_create(int size) {
@@ -440,5 +444,15 @@ int _real_epoll_pwait(int epfd, struct epoll_event *events,
 }
 #endif
 
+LIB_PRIVATE
+int _real_eventfd (int initval, int flags) {
+  REAL_FUNC_PASSTHROUGH (eventfd) (initval, flags);
+}
+
+LIB_PRIVATE
+int _real_signalfd(int fd, const sigset_t *mask, int flags) {
+  REAL_FUNC_PASSTHROUGH (signalfd) (fd, mask, flags);
+}
+
 // Used for wrappers for mmap, sbrk
 void _dmtcp_setup_trampolines() {}
