@@ -33,6 +33,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <poll.h>
 #include <fcntl.h>
 #include "constants.h"
 #include "sockettable.h"
@@ -77,11 +78,12 @@ void _dmtcp_remutex_on_fork() {
  *    then call dlsym() on the libc handle to find the addresses of the libc
  *    functions wrapped by DMTCP.
  *
- * This work great for a while until we needed wrappers for malloc/calloc/free
- * etc. The reason was the fact that dlopen/dlsym/dlerror internally call
- * calloc to allocate a small buffer. As can be seen, dlopen calls calloc which
- * goes to the DMTCP wrapper for calloc, which in turn needs to call dlopn() to
- * find the address of libc calloc and so this goes into an infinite recursion.
+ * This worked great for a while until we needed wrappers for
+ * malloc/calloc/free, etc. The reason was the fact that dlopen/dlsym/dlerror
+ * internally call calloc to allocate a small buffer. As can be seen, dlopen
+ * calls calloc which * goes to the DMTCP wrapper for calloc, which in turn
+ * needs to call dlopen() to find the address of libc calloc and so this goes
+ * into an infinite recursion.
  *
  * 2a. Libc-offsets - take 1: To counter the problems related to malloc/calloc
  *     wrappers, DMTCP was modified to not use dlopen/dlsym. Instead, a new
@@ -131,8 +133,8 @@ void _dmtcp_remutex_on_fork() {
  *    libc.so' reveals that 'pthread_cond_broadcast', 'pthread_cond_signal',
  *    and others are defined in libc.so. Thus, depending on the library load
  *    order, RTLD_NEXT might instead resolve to the libc version, which has
- *    been shown to cause problems (e.g. in the FReD module, which has wrappers
- *    around those functions).
+ *    been shown to cause problems (e.g. in the FReD record-replay plugin,
+ *    which has wrappers around those functions).
  *
  * The work around to these problems is described in the following section.
  *
@@ -338,13 +340,11 @@ void *_real_dlsym ( void *handle, const char *symbol ) {
 LIB_PRIVATE
 int _dmtcp_unsetenv( const char *name ) {
   unsetenv (name);
-  // FIXME: Fix this by getting the symbol address from libc.
-  // Another way to fix this would be to do a getenv() here and put a '\0' byte
-  // at the start of the returned value.
-  //REAL_FUNC_PASSTHROUGH ( unsetenv ) ( name );
-  char *str = (char*) getenv(name);
-  if (str != NULL) *str = '\0';
-  return 1;
+  // One can fix this by doing a getenv() here and put a '\0' byte
+  // at the start of the returned value, but that is not correct as if you do
+  // another getenv after this, it would return "", which is not the same as
+  // NULL.
+  REAL_FUNC_PASSTHROUGH ( unsetenv ) ( name );
 }
 
 LIB_PRIVATE
@@ -539,6 +539,16 @@ int _real_system ( const char *cmd )
   REAL_FUNC_PASSTHROUGH ( system ) ( cmd );
 }
 
+LIB_PRIVATE
+FILE *_real_popen(const char *command, const char *mode) {
+  REAL_FUNC_PASSTHROUGH_TYPED (FILE*, popen) (command, mode);
+}
+
+LIB_PRIVATE
+int _real_pclose(FILE *fp) {
+  REAL_FUNC_PASSTHROUGH(pclose) (fp);
+}
+
 LIB_PRIVATE
 pid_t _real_fork( void )
 {
@@ -900,11 +910,22 @@ int _real_clone ( int ( *function ) (void *), void *child_stack, int flags, void
                                       parent_tidptr, newtls, child_tidptr );
 }
 
+LIB_PRIVATE
+int _real_pthread_join(pthread_t thread, void **retval) {
+  REAL_FUNC_PASSTHROUGH_TYPED ( int, pthread_join ) ( thread, retval );
+}
+
 LIB_PRIVATE
 int _real_pthread_tryjoin_np(pthread_t thread, void **retval) {
   REAL_FUNC_PASSTHROUGH_TYPED ( int, pthread_tryjoin_np ) ( thread, retval );
 }
 
+LIB_PRIVATE
+int _real_pthread_timedjoin_np(pthread_t thread, void **retval,
+                                 const struct timespec *abstime) {
+  REAL_FUNC_PASSTHROUGH_TYPED ( int, pthread_timedjoin_np ) ( thread, retval, abstime );
+}
+
 LIB_PRIVATE
 int _real_pthread_create(pthread_t *thread, const pthread_attr_t *attr,
     void *(*start_routine)(void*), void *arg) {
@@ -933,9 +954,25 @@ int _real_shmdt (const void *shmaddr) {
   REAL_FUNC_PASSTHROUGH ( shmdt ) (shmaddr);
 }
 
+/* glibc provides two versions of shmctl: 2.0 and 2.2. For some reason, the
+ * dlsym(RTLD_NEXT,...) is getting us the 2.0 version causing the wrong
+ * function call. For i386 architecture, we need to pass IPC_64 to the system
+ * call in order to work properly. Please refer to NOTES section of shmctl
+ * manpage.
+ */
+#ifndef IPC_64
+// Taken from <linux/ipc.h>
+# define IPC_64  0x0100  /* New version (support 32-bit UIDs, bigger
+                          message sizes, etc. */
+#endif
+
 LIB_PRIVATE
 int _real_shmctl (int shmid, int cmd, struct shmid_ds *buf) {
-  REAL_FUNC_PASSTHROUGH ( shmctl ) (shmid, cmd, buf);
+#ifdef __i386__
+  REAL_FUNC_PASSTHROUGH ( shmctl ) (shmid, cmd | IPC_64, buf);
+#else
+  REAL_FUNC_PASSTHROUGH ( shmctl ) (shmid, cmd , buf);
+#endif
 }
 
 LIB_PRIVATE
@@ -1005,6 +1042,11 @@ int _real_munmap(void *addr, size_t length) {
   REAL_FUNC_PASSTHROUGH_TYPED (int, munmap) (addr, length);
 }
 
+LIB_PRIVATE
+int _real_poll(struct pollfd *fds, nfds_t nfds, int timeout) {
+  REAL_FUNC_PASSTHROUGH (poll) (fds, nfds, timeout);
+}
+
 
 LIB_PRIVATE
 int _real_epoll_create(int size) {
@@ -1030,5 +1072,16 @@ int _real_epoll_wait(int epfd, struct epoll_event *events,
 LIB_PRIVATE
 int _real_epoll_pwait(int epfd, struct epoll_event *events,
                       int maxevents, int timeout, const sigset_t *sigmask) {
-  REAL_FUNC_PASSTHROUGH (epoll_pwait) (epfd, events, maxevents, timeout, sigmask);
+  REAL_FUNC_PASSTHROUGH (epoll_pwait) (epfd, events,
+                                       maxevents, timeout, sigmask);
+}
+
+LIB_PRIVATE
+int _real_eventfd (int initval, int flags) {
+  REAL_FUNC_PASSTHROUGH (eventfd) (initval, flags);
+}
+
+LIB_PRIVATE
+int _real_signalfd (int fd, const sigset_t *mask, int flags) {
+  REAL_FUNC_PASSTHROUGH (signalfd) (fd, mask, flags);
 }
