@@ -23,7 +23,7 @@
 #define SYSCALLWRAPPERS_H
 
 #ifndef _GNU_SOURCE
-#define _GNU_SOURCE
+# define _GNU_SOURCE
 #endif
 
 #include <stdio.h>
@@ -32,6 +32,7 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <poll.h>
 #include "constants.h"
 #include <sys/ptrace.h>
 // This was needed for 64-bit SUSE LINUX Enterprise Server 9 (Linux 2.6.5):
@@ -39,8 +40,11 @@
 # include <linux/ptrace.h>
 #endif
 #include <stdarg.h>
-#include <asm/ldt.h>
-#include <stdio.h>
+#ifndef __arm__
+# include <asm/ldt.h> // Needed for 'struct user_desc' (arg 6 of __clone)
+#else
+struct user_desc {int dummy;}; /* <asm/ldt.h> is missing in Ubuntu 11.10 */
+#endif
 #include <thread_db.h>
 #include <sys/procfs.h>
 #include <syslog.h>
@@ -48,13 +52,29 @@
 #include <sys/shm.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
-#include <sys/epoll.h>
 #include <dirent.h>
 #include <unistd.h>
 #include <pwd.h>
 #include <grp.h>
 #include <netdb.h>
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+#ifdef HAVE_EPOLL_H
+# include <sys/epoll.h>
+#else
+  /* KEEP THIS IN SYNC WITH connection.h */
+# ifndef _SYS_EPOLL_H
+#  define _SYS_EPOLL_H    1
+   struct epoll_event {int dummy;};
+   /* Valid opcodes ( "op" parameter ) to issue to epoll_ctl().  */
+#  define EPOLL_CTL_ADD 1 /* Add a file decriptor to the interface.  */
+#  define EPOLL_CTL_DEL 2 /* Remove a file decriptor from the interface.  */
+#  define EPOLL_CTL_MOD 3 /* Change file decriptor epoll_event structure.  */
+# endif
+#endif
+
 void _dmtcp_setup_trampolines();
 
 #ifdef __cplusplus
@@ -136,6 +156,8 @@ LIB_PRIVATE extern __thread int thread_performing_dlopen_dlsym;
   MACRO(execlp)                             \
   MACRO(execle)                             \
   MACRO(system)                             \
+  MACRO(popen)                              \
+  MACRO(pclose)                             \
                                             \
   MACRO(signal)                             \
   MACRO(sigaction)                          \
@@ -184,22 +206,28 @@ LIB_PRIVATE extern __thread int thread_performing_dlopen_dlsym;
   MACRO(shmdt)                              \
   MACRO(shmctl)                             \
                                             \
-  MACRO(select)                             \
   MACRO(read)                               \
   MACRO(write)                              \
                                             \
+  MACRO(select)                             \
+  MACRO(poll)                               \
+                                            \
   MACRO(epoll_create)                       \
   MACRO(epoll_create1)                      \
   MACRO(epoll_ctl)                          \
   MACRO(epoll_wait)                         \
   MACRO(epoll_pwait)                        \
+  MACRO(eventfd)                            \
+  MACRO(signalfd)                           \
 //  MACRO(creat)
 //  MACRO(openat)
 
 #define FOREACH_LIBPTHREAD_WRAPPERS(MACRO)  \
   MACRO(pthread_create)                     \
   MACRO(pthread_exit)                       \
+  MACRO(pthread_join)                       \
   MACRO(pthread_tryjoin_np)                 \
+  MACRO(pthread_timedjoin_np)               \
   MACRO(pthread_sigmask)                    \
   MACRO(pthread_mutex_lock)                 \
   MACRO(pthread_mutex_trylock)              \
@@ -263,6 +291,8 @@ LIB_PRIVATE extern __thread int thread_performing_dlopen_dlsym;
 // int _real_execlp(const char *file, const char *arg, ...);
 // int _real_execle(const char *path, const char *arg, ..., char * const envp[]);
   int _real_system ( const char * cmd );
+  FILE *_real_popen(const char *command, const char *mode);
+  int _real_pclose(FILE *fp);
 
   pid_t _real_fork();
   int _real_clone ( int ( *fn ) ( void *arg ), void *child_stack, int flags, void *arg, int *parent_tidptr, struct user_desc *newtls, int *child_tidptr );
@@ -334,7 +364,10 @@ LIB_PRIVATE extern __thread int thread_performing_dlopen_dlsym;
   int _real_pthread_create(pthread_t *thread, const pthread_attr_t *attr,
       void *(*start_routine)(void*), void *arg);
   void _real_pthread_exit(void *retval) __attribute__ ((__noreturn__));
+  int _real_pthread_join(pthread_t thread, void **retval);
   int _real_pthread_tryjoin_np(pthread_t thread, void **retval);
+  int _real_pthread_timedjoin_np(pthread_t thread, void **retval,
+                                 const struct timespec *abstime);
 
   int _real_xstat(int vers, const char *path, struct stat *buf);
   int _real_xstat64(int vers, const char *path, struct stat64 *buf);
@@ -391,6 +424,8 @@ LIB_PRIVATE extern __thread int thread_performing_dlopen_dlsym;
                                    const struct timespec *abstime);
   int _real_pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex);
 
+  int _real_poll(struct pollfd *fds, nfds_t nfds, int timeout);
+
   int _real_epoll_create(int size);
   int _real_epoll_create1(int flags);
   int _real_epoll_ctl(int epfd, int op, int fd, struct epoll_event *event);
@@ -398,6 +433,8 @@ LIB_PRIVATE extern __thread int thread_performing_dlopen_dlsym;
                        int maxevents, int timeout);
   int _real_epoll_pwait(int epfd, struct epoll_event *events,
                         int maxevents, int timeout, const sigset_t *sigmask);
+  int _real_eventfd(int initval, int flags);
+  int _real_signalfd (int fd, const sigset_t *mask, int flags);
 
 #ifdef PID_VIRTUALIZATION
   pid_t _real_getpid(void);
