@@ -20,13 +20,15 @@
  ****************************************************************************/
 
 #include <sys/syscall.h>
+#include <semaphore.h>
 #include "constants.h"
 #include "dmtcpworker.h"
 #include "mtcpinterface.h"
 #include "syscallwrappers.h"
 #include "virtualpidtable.h"
-#include "dmtcpmodule.h"
+#include "dmtcpplugin.h"
 #include "uniquepid.h"
+#include "util.h"
 #include "../jalib/jassert.h"
 #include "../jalib/jalloc.h"
 
@@ -37,6 +39,7 @@ struct ThreadArg {
   void * ( *pthread_fn ) ( void *arg ); // pthread_create calls fn -> void *
   void *arg;
   pid_t original_tid;
+  sem_t sem;
 };
 
 // bool isConflictingTid( pid_t tid )
@@ -62,8 +65,10 @@ static void *pthread_start(void *arg)
   pid_t orig_tid = threadArg->original_tid;
   JASSERT(pthread_fn != 0x0);
   JALLOC_HELPER_FREE(arg); // Was allocated in calling thread in pthread_create
+  mtcpFuncPtrs.fill_in_pthread_id(_real_gettid(), pthread_self());
   dmtcp::ThreadSync::decrementUninitializedThreadCount();
   void *result = (*pthread_fn)(thread_arg);
+  WRAPPER_EXECUTION_DISABLE_CKPT();
   mtcpFuncPtrs.threadiszombie();
   /*
    * This thread has finished its execution, do some cleanup on our part.
@@ -73,6 +78,8 @@ static void *pthread_start(void *arg)
    */
   dmtcp::VirtualPidTable::instance().erase(orig_tid);
   dmtcp::VirtualPidTable::instance().eraseTid(orig_tid);
+  WRAPPER_EXECUTION_ENABLE_CKPT();
+  dmtcp::ThreadSync::unsetOkToGrabLock();
   return result;
 }
 
@@ -96,9 +103,6 @@ int clone_start(void *arg)
   pid_t tid = _real_gettid();
   JTRACE ("In clone_start");
 
-  // FIXME: Why not do this in the mtcp.c::__clone?
-  mtcpFuncPtrs.fill_in_pthread_id(tid, pthread_self());
-
   if ( dmtcp::VirtualPidTable::isConflictingPid ( tid ) ) {
     JTRACE ("TID conflict detected.  Exiting thread.");
     mtcpFuncPtrs.threadiszombie();
@@ -113,9 +117,6 @@ int clone_start(void *arg)
   int (*fn) (void *) = threadArg->fn;
   void *thread_arg = threadArg->arg;
 
-  // Free memory previously allocated through JALLOC_HELPER_MALLOC in __clone
-  JALLOC_HELPER_FREE(threadArg);
-
   if (original_tid == -1) {
     /*
      * original tid is not known, which means this thread never existed before
@@ -133,7 +134,8 @@ int clone_start(void *arg)
 
   dmtcp::VirtualPidTable::instance().updateMapping ( original_tid, tid );
 
-  JTRACE ( "Calling user function" ) (original_tid);
+  // Notify the parent thread that we are done initializing.
+  sem_post(&threadArg->sem);
 
   /* Thread finished initialization.  It's now safe for this thread to
    * participate in checkpoint.  Decrement the uninitializedThreadCount in
@@ -141,7 +143,7 @@ int clone_start(void *arg)
    */
   dmtcp::ThreadSync::decrementUninitializedThreadCount();
 
-  // return (*(threadArg->fn)) ( threadArg->arg );
+  JTRACE ( "Calling user function" ) (original_tid);
   int result = (*fn) ( thread_arg );
 
   JTRACE ( "Thread returned:" ) (original_tid);
@@ -199,7 +201,9 @@ extern "C" int pthread_create(pthread_t *thread, const pthread_attr_t *attr,
   if (threadCreationLockAcquired) {
     dmtcp::ThreadSync::threadCreationLockUnlock();
   }
-  if (retval != 0) { // if we failed to create new pthread
+  if (retval == 0) {
+    dmtcp::VirtualPidTable::instance().clearPthreadJoinState(*thread);
+  } else { // if we failed to create new pthread
     JALLOC_HELPER_FREE(threadArg);
   }
   return retval;
@@ -268,6 +272,7 @@ extern "C" int __clone(int (*fn) (void *arg), void *child_stack, int flags, void
   threadArg->fn = fn;
   threadArg->arg = arg;
   threadArg->original_tid = originalTid;
+  sem_init(&threadArg->sem, 0, 0);
 
   int tid;
 
@@ -332,25 +337,29 @@ extern "C" int __clone(int (*fn) (void *arg), void *child_stack, int flags, void
         dmtcp::VirtualPidTable::instance().updateMapping(originalTid, tid);
         dmtcp::VirtualPidTable::InsertIntoPidMapFile(originalTid, tid);
         tid = originalTid;
-      } else {
-        /* Newly created thread, insert mappings */
-        dmtcp::VirtualPidTable::instance().updateMapping(tid, tid);
       }
       break;
     }
   }
 
   if (tid == -1) {
-    // Free the memory which was previously allocated by calling
-    // JALLOC_HELPER_MALLOC inside __clone wrapper
-    JALLOC_HELPER_FREE(threadArg);
-
     // If clone() failed, decrement the uninitialized thread count
     dmtcp::ThreadSync::decrementUninitializedThreadCount();
+  } else {
+    /* Wait for child thread to finish intializing.
+     * We must let the child thread insert original->current tid in the
+     * virtualpidtable. If we don't wait for the child thread and update the
+     * pidtable ourselves, there is a possible race if the child thread is
+     * short lived. In that case, the parent thread might insert
+     * original->current mapping well after the child thread has exited causing
+     * stale entries in the virtualpidtable.
+     */
+    sem_wait(&threadArg->sem);
+    sem_destroy(&threadArg->sem);
   }
 
-  /* Release the wrapperExeution lock */
-  WRAPPER_EXECUTION_ENABLE_CKPT();
+  JALLOC_HELPER_FREE(threadArg);
+  WRAPPER_EXECUTION_ENABLE_CKPT(); /* Release the wrapperExeution lock */
 
   return tid;
 #endif
@@ -361,6 +370,7 @@ extern "C" void pthread_exit(void * retval)
   mtcpFuncPtrs.threadiszombie();
   dmtcp::VirtualPidTable::instance().erase(gettid());
   dmtcp::VirtualPidTable::instance().eraseTid(gettid());
+  dmtcp::ThreadSync::unsetOkToGrabLock();
   _real_pthread_exit(retval);
   for(;;); // To hide compiler warning about "noreturn" function
 }
@@ -386,37 +396,26 @@ extern "C" void pthread_exit(void * retval)
  *
  * Similar measures are taken for pthread_timedjoin_np().
  */
+static struct timespec ts_100ms = {0, 100 * 1000 * 1000};
 extern "C" int pthread_join(pthread_t thread, void **retval)
 {
   int ret;
+  struct timespec ts;
   if (!dmtcp::VirtualPidTable::instance().beginPthreadJoin(thread)) {
     return EINVAL;
   }
 
   while (1) {
     WRAPPER_EXECUTION_DISABLE_CKPT();
-    ret = _real_pthread_tryjoin_np(thread, retval);
+    JASSERT(clock_gettime(CLOCK_REALTIME, &ts) != -1);
+    TIMESPEC_ADD(&ts, &ts_100ms, &ts);
+    ret = _real_pthread_timedjoin_np(thread, retval, &ts);
     WRAPPER_EXECUTION_ENABLE_CKPT();
-
-    if (ret != EBUSY) {
+    if (ret != ETIMEDOUT) {
       break;
     }
-
-    const struct timespec timeout = {(time_t) 0, (long)100 * 1000 * 1000};
-    nanosleep(&timeout, NULL);
   }
 
-#ifdef PTRACE
-  /* Wrap the call to pthread_join() to make sure we call
-   * delete_thread_on_pthread_join().
-   * FIXME:  MTCP:process_pthread_join(thread) is calling threadisdead() THIS
-   *         SHOULDN'T BE NECESSARY.
-   */
-  if (ret == 0) {
-    mtcpFuncPtrs.process_pthread_join(thread);
-  }
-#endif
-
   dmtcp::VirtualPidTable::instance().endPthreadJoin(thread);
   return ret;
 }
@@ -432,17 +431,6 @@ extern "C" int pthread_tryjoin_np(pthread_t thread, void **retval)
   ret = _real_pthread_tryjoin_np(thread, retval);
   WRAPPER_EXECUTION_ENABLE_CKPT();
 
-#ifdef PTRACE
-  /* Wrap the call to pthread_join() to make sure we call
-   * delete_thread_on_pthread_join().
-   * FIXME:  MTCP:process_pthread_join(thread) is calling threadisdead() THIS
-   *         SHOULDN'T BE NECESSARY.
-   */
-  if (ret == 0) {
-    mtcpFuncPtrs.process_pthread_join(thread);
-  }
-#endif
-
   dmtcp::VirtualPidTable::instance().endPthreadJoin(thread);
   return ret;
 }
@@ -451,6 +439,7 @@ extern "C" int pthread_timedjoin_np(pthread_t thread, void **retval,
                                     const struct timespec *abstime)
 {
   int ret;
+  struct timespec ts;
   if (!dmtcp::VirtualPidTable::instance().beginPthreadJoin(thread)) {
     return EINVAL;
   }
@@ -460,40 +449,25 @@ extern "C" int pthread_timedjoin_np(pthread_t thread, void **retval,
    * the abstime provided by the caller
    */
   while (1) {
-    struct timeval tv;
-    struct timespec ts;
-    JASSERT(gettimeofday(&tv, NULL) == 0);
-    TIMEVAL_TO_TIMESPEC(&tv, &ts);
-
     WRAPPER_EXECUTION_DISABLE_CKPT();
-    ret = _real_pthread_tryjoin_np(thread, retval);
+    JASSERT(clock_gettime(CLOCK_REALTIME, &ts) != -1);
+    if (TIMESPEC_CMP(&ts, abstime, <)) {
+      TIMESPEC_ADD(&ts, &ts_100ms, &ts);
+      ret = _real_pthread_timedjoin_np(thread, retval, &ts);
+    } else {
+      ret = ETIMEDOUT;
+    }
     WRAPPER_EXECUTION_ENABLE_CKPT();
 
-    if (ret == 0) {
+    if (ret == EBUSY || ret == 0) {
       break;
     }
-
-    if (ts.tv_sec > abstime->tv_sec || (ts.tv_sec == abstime->tv_sec &&
-                                        ts.tv_nsec > abstime->tv_nsec)) {
+    if (TIMESPEC_CMP(&ts, abstime, >=)) {
       ret = ETIMEDOUT;
       break;
     }
-
-    const struct timespec timeout = {(time_t) 0, (long)100 * 1000 * 1000};
-    nanosleep(&timeout, NULL);
   }
 
-#ifdef PTRACE
-  /* Wrap the call to pthread_join() to make sure we call
-   * delete_thread_on_pthread_join().
-   * FIXME:  MTCP:process_pthread_join(thread) is calling threadisdead() THIS
-   *         SHOULDN'T BE NECESSARY.
-   */
-  if (ret == 0) {
-    mtcpFuncPtrs.process_pthread_join(thread);
-  }
-#endif
-
   dmtcp::VirtualPidTable::instance().endPthreadJoin(thread);
   return ret;
 }
