@@ -34,6 +34,31 @@
 typedef char * VA;
 #define UTIL_MAX_PATH_LEN 256
 
+#define TIMESPEC_CMP(a, b, CMP) 					      \
+  (((a)->tv_sec == (b)->tv_sec) ? 					      \
+   ((a)->tv_nsec CMP (b)->tv_nsec) : 					      \
+   ((a)->tv_sec CMP (b)->tv_sec))
+
+#define TIMESPEC_ADD(a, b, result)					      \
+  do {									      \
+    (result)->tv_sec = (a)->tv_sec + (b)->tv_sec;			      \
+    (result)->tv_nsec = (a)->tv_nsec + (b)->tv_nsec;			      \
+    if ((result)->tv_nsec >= 1000 * 1000 * 1000) {			      \
+	++(result)->tv_sec;						      \
+	(result)->tv_nsec -= 1000 * 1000 * 1000;			      \
+    }									      \
+  } while (0)
+
+#define TIMESPEC_SUB(a, b, result)					      \
+  do {									      \
+    (result)->tv_sec = (a)->tv_sec - (b)->tv_sec;			      \
+    (result)->tv_nsec = (a)->tv_nsec - (b)->tv_nsec;			      \
+    if ((result)->tv_nsec < 0) {					      \
+      --(result)->tv_sec;						      \
+      (result)->tv_nsec += 1000 * 1000 * 1000;				      \
+    }									      \
+  } while (0)
+
 namespace dmtcp
 {
   namespace Util
