@@ -22,7 +22,7 @@
 #include "dmtcpworker.h"
 #include "mtcpinterface.h"
 #include "dmtcpmessagetypes.h"
-#include "dmtcpmodule.h"
+#include "dmtcpplugin.h"
 
 // Initializing variable, theInstance, to an object of type DmtcpWorker,
 //   with DmtcpWorker constructor called with arg, enableCheckpointing = true
