@@ -68,6 +68,7 @@ __attribute__ ((visibility ("hidden")))
 __attribute__ ((visibility ("hidden")))
   char mtcp_saved_working_directory[PATH_MAX+1];
 
+#ifndef USE_PROC_MAPS
   /* These two are used by the linker script to define the beginning and end of
    * the image.
    * The '.long 0' is needed so shareable_begin>0 as the linker is too st00pid
@@ -78,6 +79,7 @@ asm (".section __shareable_begin ; .globl mtcp_shareable_begin \
 asm (".section __shareable_end   ; .globl mtcp_shareable_end \
                                  ; .long 0  ; mtcp_shareable_end:");
 asm (".text");
+#endif
 
 	/* Internal routines */
 
@@ -112,6 +114,11 @@ __attribute__ ((visibility ("hidden"))) void mtcp_restoreverything (void)
   VA new_brk;
   void (*finishrestore) (void);
 
+#ifdef USE_PROC_MAPS
+  VA mtcp_shareable_begin = mtcp_restore_begin;
+  VA mtcp_shareable_end = mtcp_restore_end;
+#endif
+
   DPRINTF("Entering mtcp_restart_nolibc.c:mtcp_restoreverything\n");
 
   /* The kernel (2.6.9 anyway) has a variable mm->brk that we should restore.
@@ -147,12 +154,16 @@ __attribute__ ((visibility ("hidden"))) void mtcp_restoreverything (void)
   }
   if (new_brk != mtcp_saved_break) {
     if (new_brk == current_brk && new_brk > mtcp_saved_break)
-      DPRINTF("new_brk == current_brk == %p\n  saved_break, %p,"
-              " is strictly smaller; data segment not extended.\n",
+      DPRINTF("new_brk == current_brk == %p\n; saved_break, %p,"
+              " is strictly smaller;\n  data segment not extended.\n",
               new_brk, mtcp_saved_break);
     else {
-      MTCP_PRINTF("error: new break (%p) != saved break  (%p)\n",
-                  current_brk, mtcp_saved_break);
+      if (new_brk == current_brk)
+        MTCP_PRINTF("error: new/current break (%p) != saved break (%p)\n",
+                    current_brk, mtcp_saved_break);
+      else
+        MTCP_PRINTF("error: new break (%p) != current break (%p)\n",
+                    new_brk, current_brk);
       mtcp_abort ();
     }
   }
@@ -179,8 +190,13 @@ __attribute__ ((visibility ("hidden"))) void mtcp_restoreverything (void)
   holebase  = mtcp_shareable_begin;
   holebase = (VA)((unsigned long int)holebase & -MTCP_PAGE_SIZE);
   // the unmaps will wipe what it points to anyway
+  // Force a hard error if any code tries to use the thread-pointer register.
+#if defined(__i386__) || defined(__x86_64__)
   asm volatile (CLEAN_FOR_64_BIT(xor %%eax,%%eax ; movw %%ax,%%fs)
 				: : : CLEAN_FOR_64_BIT(eax));
+#elif defined(__arm__)
+  mtcp_sys_kernel_set_tls(0);  /* Uses 'mcr', a kernel-mode instr. on ARM */
+#endif
 
   // so make sure we get a hard failure just in case
   // ... it's left dangling on something I want
@@ -505,6 +521,15 @@ static void readmemoryareas (void)
       if (mmappedat == MAP_FAILED) {
         DPRINTF("error %d mapping %p bytes at %p\n",
                 mtcp_sys_errno, area.size, area.addr);
+        if (mtcp_sys_errno == ENOMEM
+            /* && if not [vectors] segment:  occurs at 0xffff0000 for arm-32 */
+            && mtcp_strncmp(area.name, "[vectors]", sizeof("[vectors]"))) {
+          MTCP_PRINTF(
+           "\n**********************************************************\n"
+           "****** Received ENOMEM.  Trying to continue, but may fail.\n"
+           "****** Please run 'free' to see if you have enough swap space.\n"
+           "**********************************************************\n\n");
+        }
 	try_skipping_existing_segment = 1;
       }
       if (mmappedat != area.addr && !try_skipping_existing_segment) {
