@@ -34,6 +34,12 @@ RETRIES=2
 
 #Sleep after each program startup (sec)
 DEFAULT_S=0.3
+if sys.version_info[0] == 2 and sys.version_info[0:2] >= (2,7) and \
+    subprocess.check_output(['uname', '-p'])[0:3] == 'arm':
+  DEFAULT_S *= 2
+
+if testconfig.MTCP_USE_PROC_MAPS == "yes":
+  DEFAULT_S = 2*DEFAULT_S
 S=DEFAULT_S
 #Appears as S*SLOW in code.  If --slow, then SLOW=5
 SLOW=1
@@ -199,6 +205,8 @@ def launch(cmd):
       os.close(fd1); os.close(fd2)
     (pid, fd) = pty.fork()
     if pid == 0:
+      # Close all fds except stdin/stdout/stderr
+      os.closerange(3,1024)
       signal.alarm(300) # pending alarm inherited across exec, but not a fork
       # Problem:  pty.spawn invokes fork.  alarm() will have no effect.
       pty.spawn(cmd, master_read)
@@ -238,6 +246,8 @@ os.unsetenv('MTCP_SIGCKPT')
 #But note that dmtcp3, frisbee and gzip tests below still use gzip.
 if not VERBOSE:
   os.environ['JALIB_STDERR_PATH'] = os.devnull
+if VERBOSE:
+  print "coordinator port:  " + os.environ['DMTCP_PORT']
 
 #verify there is enough free space
 tmpfile=ckptDir + "/freeSpaceTest.tmp"
@@ -533,6 +543,7 @@ def runTest(name, numProcs, cmds):
 
 def saveResultsNMI():
   if testconfig.DEBUG == "yes":
+    # WARNING:  This can cause a several second delay on some systems.
     host = socket.getfqdn()
     if re.search("^nmi-.*.cs.wisc.edu$", host) or \
        re.search("^nmi-.*.cs.wisconsin.edu$", host):
@@ -591,17 +602,17 @@ resource.setrlimit(resource.RLIMIT_STACK, oldLimit)
 runTest("dmtcpaware1",   1, ["./test/dmtcpaware1"])
 
 PWD=os.getcwd()
-runTest("module-sleep2", 1, ["--with-module "+
-			     PWD+"/test/module/sleep1/dmtcp_sleep1hijack.so:"+
-			     PWD+"/test/module/sleep2/dmtcp_sleep2hijack.so "+
+runTest("plugin-sleep2", 1, ["--with-plugin "+
+			     PWD+"/test/plugin/sleep1/dmtcp_sleep1hijack.so:"+
+			     PWD+"/test/plugin/sleep2/dmtcp_sleep2hijack.so "+
 			     "./test/dmtcp1"])
 
-runTest("module-example-db", 2, ["--with-module "+
-			    PWD+"/test/module/example-db/dmtcp_example-dbhijack.so "+
+runTest("plugin-example-db", 2, ["--with-plugin "+
+			    PWD+"/test/plugin/example-db/dmtcp_example-dbhijack.so "+
 			     "env EXAMPLE_DB_KEY=1 EXAMPLE_DB_KEY_OTHER=2 "+
 			     "./test/dmtcp1",
-			         "--with-module "+
-			    PWD+"/test/module/example-db/dmtcp_example-dbhijack.so "+
+			         "--with-plugin "+
+			    PWD+"/test/plugin/example-db/dmtcp_example-dbhijack.so "+
 			     "env EXAMPLE_DB_KEY=2 EXAMPLE_DB_KEY_OTHER=1 "+
 			     "./test/dmtcp1"])
 
@@ -614,6 +625,11 @@ runTest("shared-fd",     2, ["./test/shared-fd"])
 
 runTest("stale-fd",      2, ["./test/stale-fd"])
 
+# Disable procfd1 until we fix readlink
+#runTest("procfd1",       2, ["./test/procfd1"])
+
+runTest("poll",          1, ["./test/poll"])
+
 runTest("forkexec",      2, ["./test/forkexec"])
 
 if testconfig.PID_VIRTUALIZATION == "yes":
@@ -630,6 +646,10 @@ os.environ['DMTCP_GZIP'] = "0"
 
 runTest("shared-memory", 2, ["./test/shared-memory"])
 
+# This is arguably a bug in the Linux kernel 3.2 for ARM.
+if sys.version_info[0] == 2 and sys.version_info[0:2] >= (2,7) and \
+    subprocess.check_output(['uname', '-p'])[0:3] == 'arm':
+  print "On ARM, there is a known issue with the sysv-shm test. Not running it."
 runTest("sysv-shm",      2, ["./test/sysv-shm"])
 
 #Invoke this test when we drain/restore data in pty at checkpoint time.
@@ -648,6 +668,7 @@ runTest("pthread2",      1, ["./test/pthread2"])
 S=3
 runTest("pthread3",      1, ["./test/pthread2 80"])
 S=DEFAULT_S
+runTest("pthread4",      1, ["./test/pthread4 80"])
 
 os.environ['DMTCP_GZIP'] = "1"
 runTest("gzip",          1, ["./test/dmtcp1"])
@@ -679,7 +700,9 @@ if testconfig.HAS_TCSH == "yes":
 
 if testconfig.HAS_ZSH == "yes":
   os.environ['DMTCP_GZIP'] = "0"
+  S=1
   runTest("zsh",         2, ["/bin/zsh -f -c 'ls; sleep 30; ls'"])
+  S=DEFAULT_S
   os.environ['DMTCP_GZIP'] = GZIP
 
 if testconfig.HAS_VIM == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
@@ -689,6 +712,8 @@ if testconfig.HAS_VIM == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
     # Delete previous vim processes.  Vim behaves poorly with stale processes.
     vimCommand = testconfig.VIM + " /etc/passwd +3" # +3 makes cmd line unique
     def killCommand(cmdToKill):
+      if os.getenv('USER') == None:
+        return
       ps = subprocess.Popen(['ps', '-u', os.environ['USER'], '-o', 'pid,command'],
     		            stdout=subprocess.PIPE).communicate()[0]
       for row in ps.split('\n')[1:]:
@@ -730,7 +755,6 @@ if testconfig.HAS_SCRIPT == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
 if testconfig.HAS_SCREEN == "yes" and testconfig.PID_VIRTUALIZATION == "yes":
   S=1
   if sys.version_info[0:2] >= (2,6):
-    host = socket.getfqdn()
     runTest("screen",    3,  ["env TERM=vt100 " + testconfig.SCREEN +
                                 " -c /dev/null -s /bin/sh"])
   S=DEFAULT_S
@@ -820,8 +844,10 @@ if testconfig.HAS_OPENMPI == "yes":
                      os.environ['PATH'])):
     oldPath = os.environ['PATH']
     os.environ += ":" + os.path.dirname(testconfig.OPENMPI_MPIRUN)
+  S=1
   runTest("openmpi", [5,6], [testconfig.OPENMPI_MPIRUN + " -np 4" +
 			     " ./test/openmpi"])
+  S=DEFAULT_S
   if oldPath:
     os.environ['PATH'] = oldPath
   if oldPath == None:
