@@ -2,7 +2,8 @@
 #include <stdio.h>
 #include <unistd.h>
 
-#include "../dmtcp/src/dmtcpaware.h"
+/* Be sure to compile with -I<path>; see Makefile in this directory. */
+#include "dmtcpaware.h"
 
 void pre(){
   printf("HOOK: preCheckpoint\n");
@@ -40,7 +41,7 @@ int main(int argc, char* argv[])
     fflush(stdout);
     ls = dmtcpGetLocalStatus();
     assert(ls!=NULL);
-    printf("{\n\t %d,\n\t %d,\n\t %s,\n\t %s}\n", 
+    printf("{\n\t %d,\n\t %d,\n\t %s,\n\t %s}\n",
         ls->numCheckpoints, ls->numRestarts, ls->checkpointFilename, ls->uniquePidStr);
 
     printf("dmtcpGetCoordinatorStatus()=");
@@ -50,7 +51,7 @@ int main(int argc, char* argv[])
     printf("{%d,%d}\n", cs->numProcesses, cs->isRunning);
     assert(cs->numProcesses>0);
 
-    //lock should be recursive 
+    //lock should be recursive
     dmtcpDelayCheckpointsLock();
     dmtcpDelayCheckpointsLock();
     dmtcpDelayCheckpointsLock();
