@@ -2,7 +2,8 @@
 #include <stdio.h>
 #include <unistd.h>
 
-#include "../dmtcp/src/dmtcpaware.h"
+/* Be sure to compile with -I<path>; see Makefile in this directory. */
+#include "dmtcpaware.h"
 
 // this example tests dmtcpDelayCheckpoins[Un]Lock()
 
@@ -12,13 +13,13 @@ int main(int argc, char* argv[])
   const DmtcpLocalStatus * ls;
   while (1)
   {
-    if(dmtcpIsEnabled()){ 
+    if(dmtcpIsEnabled()){
       ls = dmtcpGetLocalStatus();
       printf("working... %d (status: %d checkpoints / %d restarts)\n", ++count,ls->numCheckpoints, ls->numRestarts);
     }else{
       printf("working... %d\n", ++count);
     }
-    
+
     sleep(1);
 
     dmtcpDelayCheckpointsLock();
